/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.profiling;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.microej.profiling.internal.Strings;
import com.microej.profiling.internal.snapshot.Snapshot.Type;
import com.microej.profiling.internal.snapshot.ThreadInfo;
import com.microej.profiling.profiler.ComparableProfiler;
import com.microej.profiling.profiler.HeapProfiler;

import ej.basictool.ArrayTools;
import ej.bon.IllegalStateException;

/**
 *
 * A profiler that takes samples, it can manage different stack levels.
 * <p>
 * Set the {@link ComparableProfiler} to follow using {@link #addProfiler(ComparableProfiler)} function. Then set the
 * code block using {@link #enter(String)} and {@link #exit(String)}. Some intermediate laps can be added using
 * {@link #lap(String)}.
 *
 * <p>
 * Usage:
 * <ul>
 * <li>Initialize a {@link SampleProfiler}:
 * <ul>
 * <li>With one {@link ComparableProfiler}:
 *
 * <pre>
 * SampleProfiler.addProfiler(new TimeProfiler());
 * </pre>
 *
 * </li>
 * <li>Or with multiple {@link ComparableProfiler}:
 *
 * <pre>
 * List&lt;ComparableProfiler&gt; list = new ArrayList&lt;&gt;();
 * list.add(new TimeProfiler());
 * list.add(new HeapProfiler());
 * list.add(new ThreadsProfiler());
 *
 * SampleProfiler.addProfilers(list);
 * </pre>
 *
 * </li>
 * </ul>
 * </li>
 * <li>Start a section: {@link #enter(String)}, {@link #enter(Group)} or {@link #enter(String, Group)}.</li>
 * <li>Add some intermediate steps: {@link #lap(String)}.</li>
 * <li>Exit a section: {@link #exit(String)}, {@link #exit(Group)} or {@link #exit(String, Group)}.</li>
 * <li>Print the values: {@link #print()}.</li>
 * </ul>
 * Each section entered, must be exited to be printed.
 */
public class SampleProfiler {

	private static final int INITIAL_CAPACITY = 32;
	private static long THREAD_INFO_SIZE;
	// TODO manage ThreadInfos map size.
	private static final Map<Thread, ThreadInfo> ThreadInfos = new HashMap<>(INITIAL_CAPACITY);
	private static final List<ComparableProfiler> Profilers = new ArrayList<>();
	// TODO manage set size.
	private static final Set<Group> Groups = new HashSet<>(INITIAL_CAPACITY);
	private static Level LoggerLevel = Level.INFO;

	static {
		// Compute the size of a snapshot.
		Runtime runtime = Runtime.getRuntime();
		ThreadInfo[] snapshots = new ThreadInfo[0];
		snapshots = ArrayTools.add(snapshots, new ThreadInfo(Profilers, null));
		System.gc();
		long memory = runtime.freeMemory();
		snapshots = ArrayTools.add(snapshots, new ThreadInfo(Profilers, null));
		System.gc();
		THREAD_INFO_SIZE = memory - runtime.freeMemory();
	}

	private SampleProfiler() {
		// Forbid instantiation
	}

	/**
	 * Sets the log LoggerLevel.
	 *
	 * @param level
	 *            the LoggerLevel to log the message.
	 */
	public static void setLoggerLevel(Level level) {
		SampleProfiler.LoggerLevel = level;
	}

	/**
	 * Adds a collection of {@link SampleProfiler} to follow.
	 *
	 * @param profilers
	 *            the Profilers to follow.
	 * @throws IllegalStateException
	 *             when the profiler is already started (an {@link #enter(String)}, {@link #enter(Group)},
	 *             {@link #enter(String, Group)} or {@link #lap(String)} has already been called.
	 * @see #clear()
	 */
	public static void addProfilers(Collection<ComparableProfiler> profilers) throws IllegalStateException {
		synchronized (ThreadInfos) {
			synchronized (Groups) {
				if (!ThreadInfos.isEmpty() || !Groups.isEmpty()) {
					throw new IllegalStateException();
				}
				SampleProfiler.Profilers.addAll(profilers);
			}
		}
	}

	/**
	 * Adds a {@link SampleProfiler} to follow.
	 *
	 * @param profiler
	 *            the profiler to follow.
	 * @throws IllegalStateException
	 *             when the profiler is already started (an {@link #enter(String)}, {@link #enter(Group)},
	 *             {@link #enter(String, Group)} or {@link #lap(String)} has already been called.
	 * @see #clear()
	 */
	public static void addProfiler(ComparableProfiler profiler) throws IllegalStateException {
		synchronized (ThreadInfos) {
			synchronized (Groups) {
				if (!ThreadInfos.isEmpty() || !Groups.isEmpty()) {
					throw new IllegalStateException();
				}
				SampleProfiler.Profilers.add(profiler);
			}
		}
	}

	/**
	 * Clears all the Profilers registered.
	 *
	 * @throws IllegalStateException
	 *             when the profiler is already started (an {@link #enter(String)}, {@link #enter(Group)},
	 *             {@link #enter(String, Group)} or {@link #lap(String)} has already been called.
	 * @see #clear()
	 */
	public static void clearProfilers() throws IllegalStateException {
		synchronized (ThreadInfos) {
			synchronized (Groups) {
				if (!ThreadInfos.isEmpty() || !Groups.isEmpty()) {
					throw new IllegalStateException();
				}
				SampleProfiler.Profilers.clear();
			}
		}
	}

	/**
	 * Enters a new lap. The value displayed will be the comparison from the last lap, or the start of the block to the
	 * lap.
	 *
	 * <p>
	 * For example:
	 *
	 * <pre>
	 * SampleProfiler.addProfiler(new TimeProfiler());
	 * for (int i = 0; i &lt; 5; i++) {
	 * 	new Thread(new Runnable() {
	 *
	 * 		&#64;Override
	 * 		public void run() {
	 * 			f();
	 * 		}
	 *
	 * 	}).start();
	 * }
	 * Thread.sleep(5000);
	 * SampleProfiler.print();
	 * </pre>
	 *
	 * With the functions to profile:
	 *
	 * <pre>
	 * private void f() {
	 * 	SampleProfiler.enter("f()");
	 * 	try {
	 * 		g();
	 * 		SampleProfiler.lap("f");
	 * 		// Some code
	 * 		SampleProfiler.lap("2 f");
	 * 		g();
	 * 	} finally {
	 * 		SampleProfiler.exit("f()");
	 * 	}
	 * }
	 * </pre>
	 *
	 * <pre>
	 * private void g() throws InterruptedException {
	 * 	SampleProfiler.enter("g()");
	 * 	try {
	 * 		// Some code
	 * 	} finally {
	 * 		SampleProfiler.exit("g()");
	 * 	}
	 * }
	 * </pre>
	 *
	 * Will output
	 *
	 * <pre>
	 * sampleprofiler INFO:
	 * thread5:
	 * - Time:
	 * f()	312 ms
	 * 	g()	102 ms
	 * * lap f	103 ms
	 * * lap 2 f	106 ms
	 * 	g()	100 ms
	 *
	 * thread6:
	 * - Time:
	 * f()	309 ms
	 * 	g()	101 ms
	 * * lap f	102 ms
	 * * lap 2 f	105 ms
	 * 	g()	100 ms
	 *
	 * thread4:
	 * - Time:
	 * f()	310 ms
	 * 	g()	101 ms
	 * * lap f	104 ms
	 * * lap 2 f	101 ms
	 * 	g()	102 ms
	 *
	 * thread3:
	 * - Time:
	 * f()	318 ms
	 * 	g()	100 ms
	 * * lap f	105 ms
	 * * lap 2 f	100 ms
	 * 	g()	101 ms
	 *
	 * thread2:
	 * - Time:
	 * f()	311 ms
	 * 	g()	100 ms
	 * * lap f	107 ms
	 * * lap 2 f	100 ms
	 * 	g()	102 ms
	 * </pre>
	 *
	 * @param tag
	 *            the lap name.
	 */
	public static void lap(String tag) {
		tag(Thread.currentThread(), tag, Type.LAP, null);
	}

	/**
	 * Enters a new code block.
	 *
	 * <p>
	 * For example:
	 *
	 * <pre>
	 * SampleProfiler.addProfiler(new TimeProfiler());
	 * for (int i = 0; i &lt; 5; i++) {
	 * 	new Thread(new Runnable() {
	 *
	 * 		&#64;Override
	 * 		public void run() {
	 * 			f();
	 * 		}
	 *
	 * 	}).start();
	 * }
	 * Thread.sleep(5000);
	 * SampleProfiler.print();
	 * </pre>
	 *
	 * With the functions to profile:
	 *
	 * <pre>
	 * private void f() {
	 * 	SampleProfiler.enter("f()");
	 * 	try {
	 * 		g();
	 * 		// Some code
	 * 		g();
	 * 	} finally {
	 * 		SampleProfiler.exit("f()");
	 * 	}
	 * }
	 * </pre>
	 *
	 * <pre>
	 * private void g() throws InterruptedException {
	 * 	SampleProfiler.enter("g()");
	 * 	try {
	 * 		// Some code
	 * 	} finally {
	 * 		SampleProfiler.exit("g()");
	 * 	}
	 * }
	 * </pre>
	 *
	 * Will output
	 *
	 * <pre>
	 * sampleprofiler INFO:
	 * thread5:
	 * - Time:
	 * f()	316 ms
	 * 	g()	104 ms
	 * 	g()	106 ms
	 *
	 * thread4:
	 * - Time:
	 * f()	317 ms
	 * 	g()	102 ms
	 * 	g()	102 ms
	 *
	 * thread6:
	 * - Time:
	 * f()	316 ms
	 * 	g()	101 ms
	 * 	g()	107 ms
	 *
	 * thread3:
	 * - Time:
	 * f()	308 ms
	 * 	g()	103 ms
	 * 	g()	102 ms
	 *
	 * thread2:
	 * - Time:
	 * f()	311 ms
	 * 	g()	101 ms
	 * 	g()	101 ms
	 * </pre>
	 *
	 * @param tag
	 *            the code block tag.
	 */
	public static void enter(String tag) {
		enter(Thread.currentThread(), tag, null);
	}

	/**
	 * Enters a new code block.
	 *
	 * <pre>
	 * static Group groupF = new Group("groupF");
	 * static Group groupG = new Group("groupG");
	 *
	 * SampleProfiler.addProfiler(new TimeProfiler());
	 * for (int i = 0; i &lt; 5; i++) {
	 * 	new Thread(new Runnable() {
	 *
	 * 		&#64;Override
	 * 		public void run() {
	 * 			f();
	 * 		}
	 *
	 * 	}).start();
	 * }
	 * Thread.sleep(5000);
	 * SampleProfiler.print();
	 * </pre>
	 *
	 * With the functions to profile:
	 *
	 * <pre>
	 * private void f() {
	 * 	SampleProfiler.enter("f()", groupF);
	 * 	try {
	 * 		g();
	 * 		// Some code
	 * 		g();
	 * 	} finally {
	 * 		SampleProfiler.exit("f()", groupF);
	 * 	}
	 * }
	 * </pre>
	 *
	 * <pre>
	 * private void g() {
	 * 	SampleProfiler.enter("g()", groupG);
	 * 	try {
	 * 		// Some code
	 * 	} finally {
	 * 		SampleProfiler.exit("g()", groupG);
	 * 	}
	 * }
	 * </pre>
	 *
	 * Will output
	 *
	 * <pre>
	 * thread5:
	 * - Time:
	 * f()	340 ms
	 *	g()	107 ms
	 *	g()	100 ms
	 *
	 * thread3:
	 * - Time:
	 * f()	351 ms
	 * 	g()	101 ms
	 * 	g()	100 ms
	 *
	 * thread4:
	 * - Time:
	 * f()	338 ms
	 * 	g()	103 ms
	 * 	g()	111 ms
	 *
	 * thread2:
	 * - Time:
	 * f()	450 ms
	 * 	g()	208 ms
	 * 	g()	106 ms
	 *
	 * thread6:
	 * - Time:
	 * f()	338 ms
	 * 	g()	102 ms
	 * 	g()	100 ms
	 *
	 * Groups:
	 * groupF:
	 *  - Time:	count=5, min=338 ms, max=450 ms, avg=363 ms
	 *
	 * groupG:
	 *  - Time:	count=10, min=100 ms, max=208 ms, avg=114 ms
	 * </pre>
	 *
	 * @param tag
	 *            the code block tag.
	 * @param group
	 *            the group to profile.
	 */
	public static void enter(String tag, Group group) {
		if (group != null) {
			synchronized (Groups) {
				Groups.add(group);
			}
		}
		enter(Thread.currentThread(), tag, group);
	}

	/**
	 * Enters a new code block.
	 * <p>
	 * For example:
	 *
	 * <pre>
	 * static Group groupF = new Group("f()");
	 * static Group groupG = new Group("g()");
	 *
	 * SampleProfiler.addProfiler(new TimeProfiler());
	 * for (int i = 0; i &lt; 5; i++) {
	 * 	new Thread(new Runnable() {
	 *
	 * 		&#64;Override
	 * 		public void run() {
	 * 			f();
	 * 		}
	 *
	 * 	}).start();
	 * }
	 * Thread.sleep(5000);
	 * SampleProfiler.print();
	 * </pre>
	 *
	 * With the functions to profile:
	 *
	 * <pre>
	 * private void f() {
	 * 	SampleProfiler.enter(groupF);
	 * 	try {
	 * 		g();
	 * 		// Some code
	 * 		g();
	 * 	} finally {
	 * 		SampleProfiler.exit(groupF);
	 * 	}
	 * }
	 * </pre>
	 *
	 * <pre>
	 * private void g() throws InterruptedException {
	 * 	SampleProfiler.enter(groupG);
	 * 	try {
	 * 		// Some code
	 * 	} finally {
	 * 		SampleProfiler.exit(groupG);
	 * 	}
	 * }
	 * </pre>
	 *
	 * Will output
	 *
	 * <pre>
	 * thread5:
	 * - Time:
	 * f()	340 ms
	 *	g()	107 ms
	 *	g()	100 ms
	 *
	 * thread3:
	 * - Time:
	 * f()	351 ms
	 * 	g()	101 ms
	 * 	g()	100 ms
	 *
	 * thread4:
	 * - Time:
	 * f()	338 ms
	 * 	g()	103 ms
	 * 	g()	111 ms
	 *
	 * thread2:
	 * - Time:
	 * f()	450 ms
	 * 	g()	208 ms
	 * 	g()	106 ms
	 *
	 * thread6:
	 * - Time:
	 * f()	338 ms
	 * 	g()	102 ms
	 * 	g()	100 ms
	 *
	 * Groups:
	 * f():
	 *  - Time:	count=5, min=338 ms, max=450 ms, avg=363 ms
	 *
	 * g():
	 *  - Time:	count=10, min=100 ms, max=208 ms, avg=114 ms
	 * </pre>
	 *
	 * @param group
	 *            the group to profile.
	 */
	public static void enter(Group group) {
		if (group != null) {
			synchronized (Groups) {
				Groups.add(group);
			}
		}
		enter(Thread.currentThread(), (group != null) ? group.getName() : null, group);
	}

	/**
	 * Exits a code block. This function must be called for each {@link #enter(Group)}, {@link #enter(String)} or
	 * {@link #enter(String, Group)}.
	 * <p>
	 * Using a <code>try</code> and <code>finally</code> will force the use
	 *
	 * <pre>
	 * SampleProfiler.enter("");
	 * try {
	 * 	// Some code
	 * } finally {
	 * 	SampleProfiler.exit("");
	 * }
	 * </pre>
	 *
	 * @param tag
	 *            the code block name, should map the tag of the enter.
	 */
	public static void exit(String tag) {
		exit(Thread.currentThread(), tag);
	}

	/**
	 * Exits a code block. This function must be called for each {@link #enter(Group)}, {@link #enter(String)} or
	 * {@link #enter(String, Group)}.
	 * <p>
	 * Using a <code>try</code> and <code>finally</code> will force the use
	 *
	 * <pre>
	 * SampleProfiler.enter("");
	 * try {
	 * 	// Some code
	 * } finally {
	 * 	SampleProfiler.exit("");
	 * }
	 * </pre>
	 *
	 * @param group
	 *            the group, should map the group of the enter.
	 */
	public static void exit(Group group) {
		exit(Thread.currentThread(), (group == null) ? null : group.getName());
	}

	/**
	 * Exits a code block. This function must be called for each {@link #enter(Group)}, {@link #enter(String)} or
	 * {@link #enter(String, Group)}.
	 * <p>
	 * Using a <code>try</code> and <code>finally</code> will force the use
	 *
	 * <pre>
	 * SampleProfiler.enter("");
	 * try {
	 * 	// Some code
	 * } finally {
	 * 	SampleProfiler.exit("");
	 * }
	 * </pre>
	 *
	 * This function is equivalent to {@link #exit(String)} with the same tag.
	 *
	 * @param tag
	 *            the code block, should map the tag of the enter.
	 * @param group
	 *            the group, should map the group of the enter.
	 */
	public static void exit(String tag, Group group) {
		exit(Thread.currentThread(), tag);
	}

	/**
	 * Prints the information gathered.
	 */
	public static void print() {
		StringBuilder out = new StringBuilder(Strings.LINE_BREAK);
		synchronized (ThreadInfos) {
			for (ThreadInfo threadInfo : ThreadInfos.values()) {
				out.append(threadInfo.toString() + Strings.LINE_BREAK);
			}
		}
		if (!Groups.isEmpty()) {
			out.append(Strings.GROUPS + Strings.LINE_BREAK);
			synchronized (Groups) {
				for (Group group : Groups) {
					out.append(group.toString() + Strings.LINE_BREAK);
				}
			}
		}
		Logger.getLogger(SampleProfiler.class.getSimpleName()).log(LoggerLevel, out.toString());
	}

	/**
	 * Resets the register values. This does not reset the Profilers.
	 *
	 * @see #clearProfilers()
	 */
	public static void clear() {
		HeapProfiler.OFFSET = 0;
		synchronized (ThreadInfos) {
			ThreadInfos.clear();
		}
		synchronized (Groups) {
			for (Group group : Groups) {
				group.clear();
			}
			Groups.clear();
		}
	}

	private static void tag(Thread thread, String tag, Type type, Group group) {
		getThreadInfo(thread).snapshot(tag, type, group);
	}

	private static ThreadInfo getThreadInfo(Thread thread) {
		ThreadInfo threadInfo = ThreadInfos.get(thread);
		if (threadInfo == null) {
			threadInfo = new ThreadInfo(Profilers, thread);
			HeapProfiler.OFFSET += THREAD_INFO_SIZE;
			synchronized (ThreadInfos) {
				ThreadInfos.put(thread, threadInfo);
			}
		}
		return threadInfo;
	}

	private static void enter(Thread thread, String tag, Group group) {
		getThreadInfo(thread).increaseStackLevel();
		tag(thread, tag, Type.ENTER, group);
	}

	private static void exit(Thread thread, String tag) {
		tag(thread, tag, Type.EXIT, null);
		getThreadInfo(thread).decreaseStackLevel();
	}
}
