/*
 * Java 1.7
 *
 * Copyright 2017-2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.security.util;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;

/**
 * A DER format parsing helper class.
 */
@SuppressWarnings("nls")
public class DerInputStreamHelper {
	private static final int FALSE = 0;
	private static final int TRUE = 0xFF;
	private static final int DER_LENGTH_LONG_FORM = 0x80;
	private static final int DER_LENGTH_SHORT_FORM_MAX_LEN = 0x7f;

	/*
	 * These tags are taken from DerValue class, from sun.security.util. The type starts at the first byte of the
	 * encoding, and is one of these tag_* values. That may be all the type data that is needed. These tags are the
	 * "universal" tags ... they mean the same in all contexts. (Mask with 0x1f -- five bits.)
	 */
	/** Tag value indicating an ASN.1 "BOOLEAN" value. */
	public final static byte tag_Boolean = 0x01;

	/** Tag value indicating an ASN.1 "INTEGER" value. */
	public final static byte tag_Integer = 0x02;

	/** Tag value indicating an ASN.1 "BIT STRING" value. */
	public final static byte tag_BitString = 0x03;

	/** Tag value indicating an ASN.1 "OCTET STRING" value. */
	public final static byte tag_OctetString = 0x04;

	/** Tag value indicating an ASN.1 "NULL" value. */
	public final static byte tag_Null = 0x05;

	/** Tag value indicating an ASN.1 "OBJECT IDENTIFIER" value. */
	public final static byte tag_ObjectId = 0x06;

	/** Tag value including an ASN.1 "ENUMERATED" value */
	public final static byte tag_Enumerated = 0x0A;

	/** Tag value indicating an ASN.1 "UTF8String" value. */
	public final static byte tag_UTF8String = 0x0C;

	/** Tag value including a "printable" string */
	public final static byte tag_PrintableString = 0x13;

	/** Tag value including a "teletype" string */
	public final static byte tag_T61String = 0x14;

	/** Tag value including an ASCII string */
	public final static byte tag_IA5String = 0x16;

	/** Tag value indicating an ASN.1 "UTCTime" value. */
	public final static byte tag_UtcTime = 0x17;

	/** Tag value indicating an ASN.1 "GeneralizedTime" value. */
	public final static byte tag_GeneralizedTime = 0x18;

	/** Tag value indicating an ASN.1 "GenerallString" value. */
	public final static byte tag_GeneralString = 0x1B;

	/** Tag value indicating an ASN.1 "UniversalString" value. */
	public final static byte tag_UniversalString = 0x1C;

	/** Tag value indicating an ASN.1 "BMPString" value. */
	public final static byte tag_BMPString = 0x1E;

	// CONSTRUCTED seq/set
	/**
	 * Tag value indicating an ASN.1 "SEQUENCE" (zero to N elements, order is significant).
	 */
	public final static byte tag_Sequence = 0x30;

	/**
	 * Tag value indicating an ASN.1 "SEQUENCE OF" (one to N elements, order is significant).
	 */
	public final static byte tag_SequenceOf = 0x30;

	/**
	 * Tag value indicating an ASN.1 "SET" (zero to N members, order does not matter).
	 */
	public final static byte tag_Set = 0x31;

	/**
	 * Tag value indicating an ASN.1 "SET OF" (one to N members, order does not matter).
	 */
	public final static byte tag_SetOf = 0x31;

	/**
	 * Mask for explicit tagging.
	 */
	public static final byte MASK_CONTEXT = (byte) 0x080;

	/**
	 * Mask for constructed types.
	 */
	public static final byte MASK_CONSTRUCTED = (byte) 0x020;

	/**
	 * See the general contract of the <code>readFully</code> method of <code>DataInput</code>.
	 * <p>
	 * Bytes for this operation are read from the contained input stream.
	 *
	 * @param in
	 *            input stream from which to read.
	 * @param b
	 *            the buffer into which the data is read.
	 * @param off
	 *            the start offset of the data.
	 * @param len
	 *            the number of bytes to read.
	 * @exception EOFException
	 *                if this input stream reaches the end before reading all the bytes.
	 * @exception IOException
	 *                the stream has been closed and the contained input stream does not support reading after close, or
	 *                another I/O error occurs.
	 */
	public static void readFully(InputStream in, byte b[], int off, int len) throws IOException, EOFException {
		if (len < 0) {
			throw new IndexOutOfBoundsException();
		}
		int n = 0;
		while (n < len) {
			int count = in.read(b, off + n, len - n);
			if (count < 0) {
				throw new EOFException();
			}
			n += count;
		}
	}

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// read*() methods
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Reads the next DER-encoded Integer value from the specified InputStream, and returns it as an integer.
	 *
	 * @param is
	 *            the InputStream to parse
	 * @return the read value, as an int
	 * @throws DerFormatException
	 *             if the value is not correctly formatted.
	 * @throws IOException
	 *             if an I/O error occurs.
	 *
	 */
	public static int readInteger(InputStream is) throws DerFormatException, IOException {
		int length = expectTag(tag_Integer, is);

		if (length > 4) {
			throw new DerFormatException("Integer size limit reached (must be less than 5)");
		}

		int result = 0;
		int b;

		b = is.read();
		if (b == -1) {
			throw new EOFException();
		}
		length--;
		if ((b == 0) && (length > 2)) {
			// BER allows leading 0s but DER does not
			b = is.read();
			if (b == -1) {
				throw new EOFException();
			}
			if ((byte) b > 0) {
				throw new DerFormatException("Invalid encoding: redundant leading 0s");
			}
			length--;
		}
		result = b;
		while (length > 0) {
			result = result << 8;
			b = is.read();
			if (b == -1) {
				throw new EOFException();
			}
			length--;
			result |= (0xff & b);
		}
		return result;
	}

	/**
	 * Reads the next DER-encoded UTF8String value from the specified InputStream, and returns it as a String.
	 *
	 * @param is
	 *            the InputStream to parse
	 * @return the read value, as an String
	 * @throws DerFormatException
	 *             if the value is not correctly formatted.
	 * @throws IOException
	 *             if an I/O error occurs.
	 *
	 */
	public static String readUTF8String(InputStream is) throws IOException, DerFormatException {
		int length = expectTag(tag_UTF8String, is);

		byte[] retval = new byte[length];
		readFully(is, retval, 0, length);
		return new String(retval);
	}

	/**
	 * Reads the next DER-encoded value from the specified InputStream, and returns it as a String. Supported string
	 * types are:
	 * <ul>
	 * <li>UTF8String
	 * <li>IA5String
	 * <li>PrintableString
	 * </ul>
	 *
	 * @param is
	 *            the InputStream to parse
	 * @return the read value, as an String
	 * @throws DerFormatException
	 *             if the value is not correctly formatted.
	 * @throws IOException
	 *             if an I/O error occurs.
	 *
	 */
	public static String readString(InputStream is) throws IOException, DerFormatException {
		int tag = is.read();
		if (tag == -1) {
			throw new EOFException();
		}
		String encoding;
		switch (tag) {
		case tag_UTF8String:
			encoding = "UTF-8";
			break;
		case tag_PrintableString:
		case tag_IA5String:
		case tag_GeneralString:
			encoding = "US-ASCII";
			break;
		case tag_T61String:
			encoding = "ISO-8859-1";
			break;
		default:
			throw new DerFormatException("Not a supported String format: " + Integer.toHexString(tag));
		}
		int length = readDERLength(is);
		byte[] bytes = new byte[length];
		readFully(is, bytes, 0, length);
		return new String(bytes, encoding);
	}

	/**
	 * Reads the next DER-encoded Boolean value from the specified InputStream, and returns it as an boolean.
	 *
	 * @param is
	 *            the InputStream to parse
	 * @return the read value, as an boolean
	 * @throws DerFormatException
	 *             if the value is not correctly formatted.
	 * @throws IOException
	 *             if an I/O error occurs.
	 *
	 */
	public static boolean readBoolean(InputStream is) throws DerFormatException, IOException {
		int length = expectTag(tag_Boolean, is);

		if (length != 1) {
			throw new DerFormatException("Integer size limit reached (must be 1 for boolean but is " + length + ")");
		}

		int b = is.read();
		if (b == -1) {
			throw new EOFException();
		}
		if (b == FALSE) {
			return false;
		} else if (b == TRUE) {
			return true;
		} else {
			throw new DerFormatException("Illegal boolean value: " + Integer.toHexString(b & 0xFF));
		}
	}

	/**
	 * Reads the next DER-encoded ObjectID value from the specified InputStream, and returns it as a String
	 * representation in hexadecimal format.
	 * <p>
	 * This method does not check the encoding.
	 *
	 * @param is
	 *            the InputStream to parse
	 * @return the read value, as String representing the OID.
	 * @throws DerFormatException
	 *             if the value is not correctly formatted.
	 * @throws IOException
	 *             if an I/O error occurs.
	 *
	 */
	public static String readOIDString(InputStream is) throws IOException, DerFormatException {
		int length = expectTag(tag_ObjectId, is);

		StringBuffer sb = new StringBuffer(length * 3);
		int firstOctet = is.read();
		if (firstOctet == -1) {
			throw new EOFException();
		}
		int firstDigit;
		int secondDigit;
		if (firstOctet < 80) {
			firstDigit = (firstOctet / 40);
			secondDigit = (firstOctet % 40);
		} else {
			firstDigit = 2;
			secondDigit = firstOctet - 80;
		}
		sb.append(firstDigit).append('.').append(secondDigit);

		// one byte has already been consumed
		length--;
		while (length > 0) {
			int r = is.read();
			if (r == -1) {
				throw new EOFException();
			}
			length--;
			int n = 0;
			long val = 0;
			while ((r & 0x80) == 0x80) {
				if (n++ == 4) {
					throw new DerFormatException("OIDs with big integers are not supported");
				}
				r &= ~0x80;
				val = (val * 128) | r;
				r = is.read();
				if (r == -1) {
					throw new EOFException();
				}
				length--;
			}
			val = (val * 128) | r;
			sb.append('.').append(val);
		}
		return sb.toString();

	}

	/**
	 * Reads the next DER-encoded NULL value from the specified InputStream, as a null value. Obviously returns nothing.
	 *
	 * @param is
	 *            the InputStream to parse
	 * @throws DerFormatException
	 *             if the value is not correctly formatted.
	 * @throws IOException
	 *             if an I/O error occurs.
	 *
	 */
	public static void readNull(InputStream is) throws IOException, DerFormatException {
		int length = expectTag(tag_Null, is);
		if (length != 0) {
			throw new DerFormatException("Unexpected length for Null: " + Integer.toHexString(length));
		}
	}

	/**
	 * Reads the next DER-encoded BitString value from the specified InputStream, and returns it as a byte[].
	 *
	 * @param is
	 *            the InputStream to parse
	 * @return the read value, as a byte[]
	 * @throws DerFormatException
	 *             if the value is not correctly formatted.
	 * @throws IOException
	 *             if an I/O error occurs.
	 *
	 */
	public static byte[] readBitString(InputStream is) throws IOException, DerFormatException {
		int length = expectTag(tag_BitString, is);
		if (length == 0) {
			throw new DerFormatException("Invalid encoding: zero length bit string");
		}
		int numOfPadBits = is.read();
		if ((numOfPadBits < 0) || (numOfPadBits > 7)) {
			throw new DerFormatException("Invalid number of padding bits");
		}
		// the first byte indicates the number of padding bits
		length--;
		byte[] bytes = new byte[length];
		readFully(is, bytes, 0, length);
		if (numOfPadBits != 0) {
			// get rid of the padding bits
			bytes[length - 1] &= (0xff << numOfPadBits);
		}
		return bytes;
	}

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Helper methods
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Reads next byte from the InputStream and verifies if the tag matches the expected tag. If so returns length of
	 * the value, otherwise throws an IOException
	 *
	 * @param expectedTag
	 *            tag expected according to the stream format.
	 * @param stream
	 *            stream to be parsed.
	 * @return lengths of the value if the read tag matches the expected tag.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws DerFormatException
	 *             if an error occurs while parsing.
	 */
	public static int expectTag(byte expectedTag, InputStream stream) throws IOException, DerFormatException {
		if (expectedTag < 0) {
			throw new IllegalArgumentException();
		}
		int tag = stream.read();
		if (tag == -1) {
			throw new EOFException();
		}
		if (expectedTag != (byte) tag) {
			throw new DerFormatException("Expecting tag 0x" + Integer.toHexString(expectedTag)
					+ " but encountered tag 0x" + Integer.toHexString(tag) + ".");
		}
		return readDERLength(stream);
	}

	/**
	 * Reads a DER-encoded value length from an input stream. The length is encoded as follow
	 *
	 * <ul>
	 * <li>Short form: only one byte, between 0 and 127
	 * <li>Long form: 1 first byte, whose 8th bit is set to 1, and 7 other bits indicate the following number of bytes
	 * storing the length as a big endian number
	 * </ul>
	 * Limitation : since this method returns an int, the length should be store in at most 4 bytes
	 *
	 * @param in
	 *            the InputStream to read from.
	 * @return the encoded length, as an integer.
	 * @throws IOException
	 *             if an I/O error occurs.
	 * @throws DerFormatException
	 *             if an error occurs while parsing.
	 */
	public static int readDERLength(InputStream in) throws IOException, DerFormatException {
		int length = in.read();

		if (length == -1) {
			throw new EOFException();
		}
		// short form or long form length ?
		if ((length & DER_LENGTH_LONG_FORM) == DER_LENGTH_LONG_FORM) {
			int n = length & (~DER_LENGTH_LONG_FORM);
			if ((n < 1) || (n > 4)) {
				throw new DerFormatException("Incorrect length encoding on " + Integer.toHexString(n) + " bytes.");
			}
			length = 0;
			// Read n bytes to retrieve the length of the stream
			for (int i = 0; i < n; i++) {
				length <<= 8;
				int nextByte = in.read();
				if (nextByte == -1) {
					throw new EOFException();
				}
				// special case for the first byte: must be non zero
				if ((i == 0) && (nextByte == 0)) {
					throw new DerFormatException("Incorrect length encoding. Unexpected zero");
				}
				length |= nextByte;
			}
			if (length < 0) {
				throw new DerFormatException("Incorrect negative length encoding.");
			}
			if (length <= DER_LENGTH_SHORT_FORM_MAX_LEN) {
				throw new DerFormatException("Incorrect length encoding. Must use short form");
			}
		}
		return length;
	}
}
