/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.microej.wadapps.launcher.util;

import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.observable.Observer;
import ej.wadapps.admin.Application;
import ej.wadapps.admin.Application.State;
import ej.wadapps.admin.ApplicationLifecycleListener;
import ej.wadapps.admin.ApplicationsManager;
import ej.wadapps.app.Activity;
import ej.wadapps.management.ActivitiesList;
import ej.wadapps.management.ActivitiesScheduler;

/**
 * Show an application's first activity.
 */
/** package **/
class ApplicationShower implements ApplicationLifecycleListener, Observer {

	private final ApplicationsManager admin;
	private final ActivitiesList activitiesList;
	private Application waitedApplication;
	private TimerTask task;

	/**
	 * Instantiates an ApplicationShower.
	 *
	 * @param application
	 *            the application to show.
	 */
	public ApplicationShower(Application application) {
		this.waitedApplication = application;
		this.admin = ServiceLoaderFactory.getServiceLoader().getService(ApplicationsManager.class);
		this.activitiesList = ServiceLoaderFactory.getServiceLoader().getService(ActivitiesList.class);
	}

	/**
	 * Show an application if it has already some activities, or wait for one to be added.
	 *
	 * @param timeout
	 *            time before stopping to wait for the activity.
	 */
	public void show(final int timeout) {
		// listen for application state changes, in case it's stopped
		if (this.admin != null) {
			this.admin.addApplicationLifecycleListener(this);
		}
		// listen for activities list updates, in case one is registered
		if (this.activitiesList != null) {
			this.activitiesList.addObserver(this);
		}
		// Synchronised used because update may be called.
		Activity activity = null;
		synchronized (this) {
			if (this.waitedApplication != null) {
				activity = LauncherHelper.getFirstActivity(this.waitedApplication);
				// if there's already an activity from this application, show it
				if (activity != null) {
					cleanup();
				} else {
					// Stop watching at timeout.
					Timer timer = ServiceLoaderFactory.getServiceLoader().getService(Timer.class, Timer.class);
					if (timer != null) {
						this.task = new TimerTask() {

							@Override
							public void run() {
								cleanup();
							}
						};
						timer.schedule(this.task, timeout);
					}
				}
			}
		}
		show(activity);
	}

	@Override
	public void stateChanged(Application application) {
		if (application.equals(this.waitedApplication)) {
			State state = application.getState();
			if (state == State.INSTALLED || state == State.UNINSTALLED) {
				// it's been stopped, cleanup
				cleanup();
			}
		}
	}

	private void cleanup() {
		synchronized (this) {
			this.waitedApplication = null;
		}
		if (this.admin != null) {
			this.admin.removeApplicationLifecycleListener(this);
		}
		if (this.activitiesList != null) {
			this.activitiesList.deleteObserver(this);
		}
		if (this.task != null) {
			this.task.cancel();
		}
	}

	@Override
	public void update() {
		Activity foundActivity = null;
		synchronized (this) {
			if (this.waitedApplication != null) {
				for (Activity activity : this.activitiesList.getActivities()) {
					if (this.waitedApplication.equals(this.admin.getApplication(activity))) {
						// found an activity, we can stop waiting
						cleanup();
						foundActivity = activity;
						break;
					}
				}
			}
		}
		show(foundActivity);
	}

	private void show(Activity activity) {
		if (activity != null) {
			final ActivitiesScheduler activitiesScheduler = ServiceLoaderFactory.getServiceLoader()
					.getService(ActivitiesScheduler.class);
			if (activitiesScheduler != null) {
				activitiesScheduler.show(activity);
			}
		}
	}
}