/*
 * Java
 *
 * Copyright 2015-2021 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.microej.wadapps.rcommand;

import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.microej.wadapps.metadata.DefaultApplicationMetadata;
import com.microej.wadapps.rcommand.util.BasicImageInfo;

import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.rcommand.CommandReader;
import ej.rcommand.CommandSender;
import ej.wadapps.admin.Application;
import ej.wadapps.admin.Application.State;
import ej.wadapps.admin.ApplicationOperationException;
import ej.wadapps.admin.ApplicationsManager;
import ej.wadapps.management.ApplicationMetadataProvider;
import ej.wadapps.management.ExtraMetadata;
import ej.wadapps.management.ImageInfo;
import ej.wadapps.storage.Storage;

/**
 * A command allowing to install an application with more parameters.
 */
@SuppressWarnings("nls")
public class InstallApplicationCommand extends AbstractCommand {

	private final ApplicationsManager applicationsManager;
	private final Logger logger;

	/**
	 * Constructs an InstallApplicationCommand.
	 */
	public InstallApplicationCommand() {
		this.applicationsManager = ServiceLoaderFactory.getServiceLoader().getService(ApplicationsManager.class);
		this.logger = Logger.getLogger(InstallApplicationCommand.class.getSimpleName());
	}

	@Override
	public String getManagedCommand() {
		return CommandConstants.INSTALL_APPLICATION_PLUS;
	}

	@Override
	protected void commandReceived(CommandReader commandReader, CommandSender commandSender) {
		try {
			String applicationId = commandReader.readString();
			String applicationName = commandReader.readString();
			String applicationVersion = commandReader.readString();
			String applicationDescription = commandReader.readString();
			this.logger.info("Receive an application " + applicationName);
			deleteOldApplication(applicationId, applicationName, applicationVersion);

			int iconCount = commandReader.readInt();
			ImageInfo[] icons = new ImageInfo[iconCount];
			Storage storage = ServiceLoaderFactory.getServiceLoader().getService(Storage.class);

			for (int i = 0; i < iconCount; i++) {
				BasicImageInfo icon = new BasicImageInfo();
				icon.setIdentifier("tmp_icon_" + commandReader.readString());
				icon.setWidth(commandReader.readInt());
				icon.setHeight(commandReader.readInt());
				storage.store(icon.getIdentifier(), commandReader.readInputStream());
				icons[i] = icon;
			}

			ApplicationMetadataProvider applicationMetadataProvider = ServiceLoaderFactory.getServiceLoader()
					.getService(ApplicationMetadataProvider.class);
			DefaultApplicationMetadata metadata = new DefaultApplicationMetadata(applicationId, icons,
					new ImageInfo[0], new ExtraMetadata[0]);
			metadata.setName(applicationName);
			metadata.setVersion(applicationVersion);
			metadata.setDescription(applicationDescription);

			try (InputStream applicationBinary = commandReader.readInputStream()) {
				try {
					this.logger.info("Install the application " + applicationName);
					final Application application = this.applicationsManager.install(applicationBinary, applicationId,
							applicationVersion);
					this.logger.info("Install the application Done " + applicationName);

					if (applicationMetadataProvider != null) {
						applicationMetadataProvider.setApplicationMetadata(application, metadata);
					}
				} finally {
					// skip any remaining bytes in the application binary stream
					int read = applicationBinary.read();
					while (read != -1) {
						read = applicationBinary.read();
					}
				}
				commandSender.startCommand(CommandConstants.INSTALL_APPLICATION_PLUS_SUCCESS);
				commandSender.flushCommand();
			}
		} catch (UnsupportedOperationException | SecurityException | ApplicationOperationException | IOException e) {
			this.logger.log(Level.SEVERE, e.getMessage(), e);
			commandSender.startCommand(CommandConstants.INSTALL_APPLICATION_PLUS_ERROR);
			if (e.getMessage() != null) {
				commandSender.sendString(e.getMessage());
			} else {
				commandSender.sendString("board internal error");
			}
			commandSender.flushCommand();
		}
	}

	private void deleteOldApplication(String applicationId, String applicationName, String applicationVersion)
			throws IllegalStateException, SecurityException, ApplicationOperationException {
		for (Application application : this.applicationsManager.getInstalledApplications()) {
			if (application.getIdentifier().equals(applicationId)) {
				State state = application.getState();
				if (state.equals(State.STARTED)) {
					this.logger.info("Stop an old application " + applicationName);
					application.stop();
				}

				this.logger.info("Uninstall an old application " + applicationName);
				application.uninstall();
			}
		}
	}
}
