/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.aws.iot.pubsub;

import java.util.Hashtable;
import java.util.Properties;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttMessage;

/**
 * A specific AWS IoT Core client, based on Paho.
 */
public class Client {

	private MqttClient mqttClient;
	private MqttCallback callback;
	private final Hashtable<String, TopicListener> listeners = new Hashtable<>();

	/**
	 * Returns the state of the connection.
	 *
	 * @return true if connected, false otherwise
	 */
	public boolean isConnected() {

		if (this.mqttClient == null) {
			return false;
		}

		return this.mqttClient.isConnected();
	}

	/**
	 * Use the connection options to open a connection to the AWS IoT services.
	 *
	 * @param connectOptions
	 *            the specific AWS IoT options
	 * @throws IotCoreException
	 *             when already connected or on communication errors
	 */
	public void connect(ConnectOptions connectOptions) throws IotCoreException {

		if (isConnected()) {
			throw new IotCoreException("Already connected.");
		}

		try {

			MqttConnectOptions options = new MqttConnectOptions();

			options.setMqttVersion(org.eclipse.paho.client.mqttv3.MqttConnectOptions.MQTT_VERSION_3_1_1);
			options.setConnectionTimeout(60);
			options.setKeepAliveInterval(60);
			Properties sslProps = new Properties();
			sslProps.setProperty("com.ibm.ssl.protocol", "TLSv1.2");
			options.setSSLProperties(sslProps);
			options.setSocketFactory(connectOptions.getSslContext().getSocketFactory());


			this.mqttClient = new MqttClient(connectOptions.buildServerAddress(), connectOptions.getThingId());

			this.callback = new MqttCallback() {

				@Override
				public void messageArrived(String topic, MqttMessage message) throws Exception {
					if (listeners.containsKey(topic)) {
						listeners.get(topic).onDataReceived(topic, message.getPayload());
					}
				}

				@Override
				public void deliveryComplete(IMqttDeliveryToken token) {
					// only supporting QoS0
					// in QoS 0 this method is only called when written in the underlying socket
					// should never fails
					// so we do nothing here
				}

				@Override
				public void connectionLost(Throwable cause) {
					// the pub/sub methods check the connection state
					// just inform of the lost connection
					cause.printStackTrace();

				}
			};

			this.mqttClient.setCallback(this.callback);

			this.mqttClient.connect(options);

		} catch (Throwable e) {
			throw new IotCoreException(e);
		}
	}

	/**
	 * Disconnect this client from the underlying communication layer.
	 *
	 * @throws IotCoreException
	 *             when not connected
	 */
	public void disconnect() throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException("Client not connected.");
		}

		try {
			this.mqttClient.disconnect();
		} catch (Throwable t) {
			throw new IotCoreException(t);
		}

	}

	/**
	 * Subscribe to a topic.
	 *
	 * @param topic
	 *            the topic you want to listened to
	 * @throws IotCoreException
	 *             when not connected and when no TopicListener was installed
	 */
	public void subscribe(String topic) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException("Client not connected.");
		}

		if (this.listeners == null) {
			throw new IotCoreException("No TopicListener installed.");
		}

		try {
			this.mqttClient.subscribe(topic, 0);
		} catch (Throwable t) {
			throw new IotCoreException(t);
		}
	}

	/**
	 * Unsubscribe from a topic.
	 *
	 * @param topic
	 *            the topic you want to stop listening to
	 * @throws IotCoreException
	 *             when not connected or on communication error
	 */
	public void unsubscribe(String topic) throws IotCoreException {
		if (!isConnected()) {
			throw new IotCoreException("Client not connected.");
		}

		try {
			this.mqttClient.unsubscribe(topic);
		} catch (Throwable t) {
			throw new IotCoreException(t);
		}
	}

	/**
	 * Publishes a message on a topic.
	 *
	 * @param topic
	 *            the topic you want to publish on
	 * @param data
	 *            the data you want to publish
	 * @throws IotCoreException
	 *             when not connected or on communication error
	 */
	public void publish(String topic, byte[] data) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException("Client not connected.");
		}

		try {
			this.mqttClient.publish(topic, data, 0, false);
		} catch (Throwable t) {
			throw new IotCoreException(t);
		}
	}

	/**
	 * Adds a topic listener on a topic. Only one listener per topic can exist, if a
	 * listener is added on a topic that already has a listener, it will be
	 * replaced.
	 *
	 * @param topic
	 *            the topic to add the listener on, one listener is allowed per
	 *            topic
	 * @param listener
	 *            the TopicListener that will process the received data
	 * @throws IotCoreException
	 *             when not connected or on communication error
	 */
	public void addTopicListener(String topic, TopicListener listener) throws IotCoreException {

		if (!isConnected()) {
			throw new IotCoreException("Client not connected.");
		}

		try {
			this.mqttClient.subscribe(topic, 0);
			this.listeners.put(topic, listener);
		} catch (Throwable t) {
			throw new IotCoreException(t);
		}
	}
}
