/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Stack;

import ej.basedriver.Controller;
import ej.basedriver.zwave.frame.AddNodeToNetwork;
import ej.basedriver.zwave.frame.ControlFrame;
import ej.basedriver.zwave.frame.RemoveNodeToNetwork;
import ej.basedriver.zwave.frame.RequestFrame;
import ej.basedriver.zwave.frame.SendData;
import ej.basedriver.zwave.util.Queue;

public class ZwaveSendThread extends Thread {
	private OutputStream outputStream;
	private SendState currentState;
	private final Stack<ZwaveNode> awakeNodes;
	private final List<SendData> wakeupQueue;
	private final Queue<RequestFrame> queue;
	private ZwaveSession session;
	private boolean stopped = false;

	public ZwaveSendThread() {
		setName(ZwaveSendThread.class.getSimpleName());
		currentState = SendState.DEFAULT;
		queue = new Queue<>(64, true);
		awakeNodes = new Stack<>();
		wakeupQueue = new ArrayList<>();
	}

	public void setOutputStream(OutputStream outputStream) {
		this.outputStream = outputStream;
	}

	public void setZwaveSession (ZwaveSession session) {
		this.session = session;
	}

	/**
	 * @param frame
	 * @return true
	 * @throws InterruptedException
	 */
	public boolean enqueue(RequestFrame frame) throws InterruptedException {
		return queue.push(frame);
	}

	/**
	 * @return the queue
	 */
	public Queue<RequestFrame> getQueue() {
		return queue;
	}

	/**
	 * @return the wakeupQueue
	 */
	public List<SendData> getWakeupQueue() {
		return wakeupQueue;
	}

	/**
	 * @return the awakeNodes
	 */
	public Stack<ZwaveNode> getAwakeNodes() {
		return awakeNodes;
	}

	/**
	 * @return the session
	 */
	public ZwaveController getController() {
		return session.getController();
	}

	@Override
	public void run() {
		RequestFrame frame = null;
		try {
			while (!stopped) {
				frame = currentState.getRequestFrame(this);

				if (frame == null) {
					continue;
				}

				if (frame instanceof SendData) {
					SendData sendFrame = (SendData) frame;
					if (! sendFrame.getNode().isReady()) {
						wakeupQueue.add(sendFrame);
						continue;
					}
				}

				session.session(frame, outputStream);
			}
		} catch (IOException | InterruptedException e) {
		}

		queue.stop();
		try {
			outputStream.close();
		} catch (IOException e) {
		}

		ZwaveController controller = this.session.getController();
		if (controller.isStarted()) {
			this.session.getController().stop();
		}
	}

	public void halt() {
		queue.stop();
		stopped = true;
		interrupt();
	}

	public synchronized void setState (SendState state) {
		this.currentState = state;
	}

	public void sendAck() throws IOException {
		ControlFrame.ACK_FRAME.send(outputStream);
	}

	public void sendNak() throws IOException {
		ControlFrame.NAK_FRAME.send(outputStream);
	}

	private interface RequestFrameProvider {
		RequestFrame getRequestFrame (ZwaveSendThread context) throws InterruptedException;
	}

	private enum SendState implements RequestFrameProvider {
		DEFAULT {
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) throws InterruptedException {

				RequestFrame pop = context.getQueue().pop();
				return pop;
			}
		},
		WAKEUP {
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) {
				Stack<ZwaveNode> awakeNodes = context.getAwakeNodes();
				try {
					ZwaveNode node = awakeNodes.firstElement();
					RequestFrame frame = getWakeupNodeCommand(context, node);
					if (frame == null) {
						synchronized (awakeNodes) {
							awakeNodes.remove(node);
							node.setAwake(false);
							//TODO send no more information

							if (awakeNodes.size() == 0) {
								context.setState(DEFAULT);
							}
						}
					}

					return frame;
				} catch (NoSuchElementException e) {
					context.setState(DEFAULT);
					return null;
				}
			}

			private RequestFrame getWakeupNodeCommand (ZwaveSendThread context, ZwaveNode nodeAwake) {
				Iterator<SendData> iterator = context.getWakeupQueue().iterator();
				while (iterator.hasNext()) {
					SendData frame = iterator.next();
					if (frame.getNode().equals(nodeAwake)) {
						iterator.remove();
						return frame;
					}
				}

				return null;
			}
		},

		INCLUSION{
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) {
				RequestFrame frame = AddNodeToNetwork.inclusionStart(context.getController());
				context.setState(INCLUSION_CLEAN);
				return frame;
			}
		},
		INCLUSION_CLEAN{
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) {
				RequestFrame frame = AddNodeToNetwork.inclusionStop(context.getController());
				context.setState(INCLUSION_STOP);
				return frame;
			}
		},
		INCLUSION_STOP{
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) {
				ZwaveController controller = context.getController();
				RequestFrame frame = AddNodeToNetwork.inclusionStop(controller);
				context.reset();
				controller.setState(Controller.RUNNING);
				return frame;
			}
		},
		EXCLUSION{
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) {
				RequestFrame frame = RemoveNodeToNetwork.exclusionStart(context.getController());
				context.setState(EXCLUSION_CLEAN);
				return frame;
			}
		},
		EXCLUSION_CLEAN{
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) {
				RequestFrame frame = RemoveNodeToNetwork.exclusionStop(context.getController());
				context.setState(EXCLUSION_STOP);
				return frame;
			}
		},
		EXCLUSION_STOP{
			@Override
			public RequestFrame getRequestFrame(ZwaveSendThread context) {
				ZwaveController controller = context.getController();
				RequestFrame frame = RemoveNodeToNetwork.exclusionStop(controller);
				context.reset();
				controller.setState(Controller.RUNNING);
				return frame;
			}
		}
	}

	public void reset () {
		this.setState(SendState.WAKEUP);
	}

	public void addAwakeNode(ZwaveNode node) {
		synchronized (awakeNodes) {
			if (! awakeNodes.contains(node)) {
				awakeNodes.push(node);
				setState(SendState.WAKEUP);
				queue.notifyReader();
			}
		}
	}

	public void inclusionStart () {
		setState(SendState.INCLUSION);
		queue.notifyReader();
	}

	public void exclusionStart () {
		setState(SendState.EXCLUSION);
		queue.notifyReader();
	}

	public void inclusionStop () {
		if (currentState == SendState.INCLUSION_CLEAN) {
			Session.NODE_FOUND.close();
			Session.ADD_NODE.close();
			queue.notifyReader();
		}
	}

	public void exclusionStop () {
		if (currentState == SendState.EXCLUSION_CLEAN) {
			Session.NODE_FOUND.close();
			Session.REMOVE_NODE.close();
			queue.notifyReader();
		}
	}
}
