/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave.device;

import java.io.IOException;

import ej.basedriver.BinaryState;
import ej.basedriver.SwitchWithReturnState;
import ej.basedriver.zwave.ZwaveNode;
import ej.basedriver.zwave.ZwaveProtocol;
import ej.basedriver.zwave.frame.SendData;
import ej.basedriver.zwave.frame.SendDataListener;

public class ZwaveSwitch extends ZwaveDevice implements SwitchWithReturnState, SendDataListener {
	public static final byte SWITCH_OFF = 0x00;
	public static final byte SWITCH_ON = (byte) 0xFF;
	private int state;
	private boolean changing = false;

	public ZwaveSwitch(ZwaveNode node) {
		super(node);
		state = BinaryState.UNKNOWN;
	}

	@Override
	public void on() throws IOException {
		sendNewState(SWITCH_ON);
	}

	@Override
	public void off() throws IOException {
		sendNewState(SWITCH_OFF);
	}

	private void sendNewState (byte state) throws IOException {
		SendData frame = new SendData(this, ZwaveProtocol.COMMAND_CLASS_SWITCH_BINARY, ZwaveProtocol.SWITCH_BINARY_SET, 1);
		frame.addParameter(state);
		node.getController().sendFrame(frame);
	}

	@Override
	public void notifyComplete() {
		if (changing) {
			changing = false;
			try {
				node.getController().getListener().onEvent(SwitchWithReturnState.class, this);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public void notifyError(RequestFrameError error) {
		state = BinaryState.UNKNOWN;
		try {
			node.getController().getListener().onError(SwitchWithReturnState.class, this);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public ZwaveNode getNode() {
		return node;
	}

	@Override
	public int getLastKnownState() {
		return state;
	}

	@Override
	public void requestStateUpdate() throws IOException {
		SendData frame = new SendData(this, ZwaveProtocol.COMMAND_CLASS_SWITCH_BINARY, ZwaveProtocol.SWITCH_BINARY_GET);
		node.getController().sendFrame(frame);
	}

	/**
	 * @param newState the new state.
	 */
	public void setState(int newState) {
		if (state != newState) {
			this.state = newState;
			changing = true;
		}
	}

	@Override
	public void callbackComplete(byte[] bytes, int commandClassIndex) {
		byte command = bytes[commandClassIndex + 1];
		if (command == ZwaveProtocol.BASIC_SET) {
			byte state = bytes[commandClassIndex + 2];
			if (state == SWITCH_OFF) {
				setState(BinaryState.OFF);
			} else if (state == SWITCH_ON) {
				setState(BinaryState.ON);
			}
		}
	}
}