/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave.device;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import ej.basedriver.Thermostat;
import ej.basedriver.ThermostatMode;
import ej.basedriver.zwave.ZwaveNode;
import ej.basedriver.zwave.ZwaveProtocol;
import ej.basedriver.zwave.frame.SendData;
import ej.basedriver.zwave.frame.SendDataListener;

/**
 *
 */
public class ZwaveThermostat extends ZwaveDevice implements Thermostat, SendDataListener {

	private enum STATE {
		INIT, REGISTERED
	};

	private final Map<Integer, ZwaveThermostatMode> modes;
	private ThermostatMode mode;
	private STATE state;

	/**
	 * @param node
	 * @throws IOException
	 */
	public ZwaveThermostat(ZwaveNode node) throws IOException {
		super(node);
		modes = new HashMap<Integer, ZwaveThermostatMode>();
		state = STATE.INIT;
		requestModesUpdate();
		// The registration will be done once the requestModes is done and received.
	}

	/**
	 * Update the list of modes.
	 */
	public void requestModesUpdate() throws IOException {
		SendData frame = new SendData(this, ZwaveProtocol.COMMAND_CLASS_THERMOSTAT_MODE,
				ZwaveProtocol.THERMOSTAT_MODE_SUPPORTED_GET, 0);
		node.getController().sendFrame(frame);

	}

	@Override
	public ThermostatMode[] getSupportedModes() {
		Collection<ZwaveThermostatMode> values = modes.values();
		return values.toArray(new ThermostatMode[values.size()]);
	}

	@Override
	public void setMode(ThermostatMode mode) throws IOException {
		if (!modes.containsValue(mode)) {
			return;
		}
		ZwaveThermostatMode zwaveMode = (ZwaveThermostatMode) mode;
		sendSetMode(zwaveMode);
	}

	/**
	 * Send the new mode.
	 *
	 * @param zwaveMode
	 * @throws IOException
	 */
	private void sendSetMode(ZwaveThermostatMode zwaveMode) throws IOException {
		SendData frame = new SendData(this, ZwaveProtocol.COMMAND_CLASS_THERMOSTAT_MODE,
				ZwaveProtocol.THERMOSTAT_MODE_SET, 1);
		frame.addParameter(zwaveMode.getModeId());
		node.getController().sendFrame(frame);

	}

	@Override
	public ThermostatMode getLastknownMode() {
		return mode;
	}

	@Override
	public void requestUpdate() throws IOException {
		SendData frame = new SendData(this, ZwaveProtocol.COMMAND_CLASS_THERMOSTAT_MODE,
				ZwaveProtocol.THERMOSTAT_MODE_GET, 0);
		node.getController().sendFrame(frame);
	}

	@Override
	public void notifyComplete() {

	}

	@Override
	public void notifyError(RequestFrameError error) {
		try {
			node.getController().getListener().onError(ZwaveThermostat.class, this);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	@Override
	public ZwaveNode getNode() {
		return node;
	}

	@Override
	public void callbackComplete(byte[] bytes, int commandClassIndex) {
		byte command = bytes[commandClassIndex + 1];
		switch (command) {
		case ZwaveProtocol.THERMOSTAT_MODE_SUPPORTED_REPORT:
			setModes(bytes, commandClassIndex + 2);
			break;
		case ZwaveProtocol.THERMOSTAT_MODE_REPORT:
			setMode(bytes, commandClassIndex);
			break;
		}

	}

	/**
	 * Registers the thermostat to the controller.
	 */
	private void register() {
		state = STATE.REGISTERED;
		node.getController().getListener().onRegister(Thermostat.class, this);
		try {
			requestUpdate();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param bytes the mode.
	 * @param commandClassIndex the command class Index.
	 */
	public void setMode(byte[] bytes, int commandClassIndex) {
		ZwaveThermostatMode zwaveThermostatMode = getMode(Integer.valueOf(bytes[commandClassIndex + 2] & 0b00011111));
		if (zwaveThermostatMode != mode) {
			this.mode = zwaveThermostatMode;
			try {
				node.getController().getListener().onEvent(Thermostat.class, this);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public ZwaveThermostatMode getMode(Integer modeId){
		return modes.get(modeId);
	}

	/**
	 * Set the modes available.
	 *
	 * @param bytes
	 *            bitmask.
	 * @param start
	 *            start of the bite mask.
	 */
	public synchronized void setModes(byte[] bytes, int start) {
		if (state == STATE.INIT) {
			modes.clear();
			if (start < bytes.length) {
				byte bitMask = bytes[start];
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_1_MODE_AUTO) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_AUTO),
							new ZwaveThermostatMode(this, ThermostatMode.AUTO, ZwaveProtocol.THERMOSTAT_MODE_AUTO));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_1_MODE_AUXILIARY) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_AUXILIARY),
							new ZwaveThermostatMode(this, ThermostatMode.AUXILIARY,
									ZwaveProtocol.THERMOSTAT_MODE_AUXILIARY));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_1_MODE_COOL) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_COOL),
							new ZwaveThermostatMode(this, ThermostatMode.COOL, ZwaveProtocol.THERMOSTAT_MODE_COOL));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_1_MODE_FAN) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_FAN),
							new ZwaveThermostatMode(this, ThermostatMode.FAN, ZwaveProtocol.THERMOSTAT_MODE_FAN));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_1_MODE_FURNACE) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_FURNACE),
							new ZwaveThermostatMode(this, ThermostatMode.FURNACE,
									ZwaveProtocol.THERMOSTAT_MODE_FURNACE));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_1_MODE_HEAT) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_HEAT),
							new ZwaveThermostatMode(this, ThermostatMode.HEAT, ZwaveProtocol.THERMOSTAT_MODE_HEAT));
				}
			}
			start++;
			if (start < bytes.length) {
				byte bitMask = bytes[start];
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_2_MODE_AUTO_CHANGEOVER) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_AUTO_CHANGEOVER),
							new ZwaveThermostatMode(this, ThermostatMode.AUTO_CHANGEOVER,
									ZwaveProtocol.THERMOSTAT_MODE_AUTO_CHANGEOVER));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_2_MODE_AWAY) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_AWAY),
							new ZwaveThermostatMode(this, ThermostatMode.AWAY, ZwaveProtocol.THERMOSTAT_MODE_AWAY));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_2_MODE_DRY) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_DRY),
							new ZwaveThermostatMode(this, ThermostatMode.DRY, ZwaveProtocol.THERMOSTAT_MODE_DRY));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_2_MODE_ENERGY_COOL) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_COOL),
							new ZwaveThermostatMode(this, ThermostatMode.ENERGY_COOL,
									ZwaveProtocol.THERMOSTAT_MODE_COOL));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_2_MODE_ENERGY_HEAT) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_ENERGY_HEAT),
							new ZwaveThermostatMode(this, ThermostatMode.ENERGY_HEAT,
									ZwaveProtocol.THERMOSTAT_MODE_ENERGY_HEAT));
				}
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_2_MODE_MOIST) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_MOIST),
							new ZwaveThermostatMode(this, ThermostatMode.MOIST, ZwaveProtocol.THERMOSTAT_MODE_MOIST));
				}
			}
			start++;
			if (start < bytes.length) {
				byte bitMask = bytes[start];
				if ((bitMask & ZwaveProtocol.THERMOSTAT_BITMASK_3_MODE_FULL_POWER) > 0) {
					modes.put(Integer.valueOf(ZwaveProtocol.THERMOSTAT_MODE_FULL_POWER),
							new ZwaveThermostatMode(this, ThermostatMode.FULL_POWER,
									ZwaveProtocol.THERMOSTAT_MODE_FULL_POWER));
				}
			}
			// The init is finished.
			register();
		}
	}
}
