/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave.device;

import java.io.IOException;

import ej.basedriver.MultilevelValue;
import ej.basedriver.Thermostat;
import ej.basedriver.ThermostatMode;
import ej.basedriver.zwave.ZwaveNode;
import ej.basedriver.zwave.ZwaveProtocol;
import ej.basedriver.zwave.frame.SendData;
import ej.basedriver.zwave.frame.SendDataListener;
import ej.basedriver.zwave.util.NumberUtil;

/**
 *
 */
public class ZwaveThermostatMode implements ThermostatMode, SendDataListener {

	private double value;
	private final int type;
	private final byte modeId;
	private final ZwaveThermostat thermostat;

	/**
	 * A thermostat mode on the Zwave protocol.
	 *
	 * @param thermostat
	 *            The node.
	 * @param type
	 *            The type of mode.
	 * @param modeId
	 *            The Zwave modeId.
	 */
	public ZwaveThermostatMode(ZwaveThermostat thermostat, int type, byte modeId) {
		this.type = type;
		this.modeId = modeId;
		this.thermostat = thermostat;
		value = MultilevelValue.UNKNOWN;
		try {
			requestUpdate();
		} catch (IOException e) {
		}
	}

	@Override
	public void setValue(double value) throws IOException {
		int precision = 1;
		byte[] byteValue = NumberUtil.writeInt(value, precision);
		int size = getUtilData(byteValue);
		SendData frame = new SendData(this, ZwaveProtocol.COMMAND_CLASS_THERMOSTAT_SETPOINT,
				ZwaveProtocol.THERMOSTAT_SETPOINT_SET, 2 + size);
		byte parameter = (byte) (getModeId() & 0x0F);
		frame.addParameter(parameter);
		parameter = (byte) ((byte) (precision << 5) | (size & 0x0F));
		frame.addParameter(parameter);
		for (int i = byteValue.length - size; i < byteValue.length; i++) {
			frame.addParameter(byteValue[i]);
		}
		thermostat.getNode().getController().sendFrame(frame);
		updateValue(value);
	}

	/**
	 * @param byteValue
	 * @return
	 */
	private int getUtilData(byte[] byteValue) {
		int utilData = 1;
		int nextStep = 2;
		int i = byteValue.length - 2;
		while (i >= 0) {
			if (byteValue[i] != 0) {
				utilData = nextStep;
				i = byteValue.length - nextStep;
				nextStep = nextStep << 1;
			}
			i--;
		}
		return utilData;
	}

	/**
	 * Update the value and send notification.
	 *
	 * @param value
	 */
	public void updateValue(double value) {
		if (this.value != value) {
			this.value = value;
			try {
				thermostat.getNode().getController().getListener().onEvent(Thermostat.class, thermostat);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public double getLastKnownValue() {
		return value;
	}

	@Override
	public void requestUpdate() throws IOException {
		SendData frame = new SendData(this, ZwaveProtocol.COMMAND_CLASS_THERMOSTAT_SETPOINT,
				ZwaveProtocol.THERMOSTAT_SETPOINT_GET, 1);
		frame.addParameter((byte) (this.getModeId() & 0x0F));
		thermostat.getNode().getController().sendFrame(frame);
	}

	@Override
	public int getType() {
		return type;
	}

	/**
	 * Gets the modeId.
	 * @return the modeId.
	 */
	public byte getModeId() {
		return modeId;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + modeId;
		result = prime * result + type;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		ZwaveThermostatMode other = (ZwaveThermostatMode) obj;
		if (modeId != other.modeId) {
			return false;
		}
		if (type != other.type) {
			return false;
		}
		return true;
	}



	@Override
	public void notifyComplete() {
		// TODO Auto-generated method stub

	}

	@Override
	public void notifyError(RequestFrameError error) {
		try {
			thermostat.getNode().getController().getListener().onError(ZwaveThermostat.class, thermostat);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public ZwaveNode getNode() {
		return thermostat.getNode();
	}

	@Override
	public void callbackComplete(byte[] bytes, int commandClassIndex) {
		byte command = bytes[commandClassIndex + 1];
		switch (command) {
		case ZwaveProtocol.THERMOSTAT_SETPOINT_REPORT:
			int precision = ((bytes[commandClassIndex + 3] & 0b11100000) >> 5);
			int size = (bytes[commandClassIndex + 3] & 0b00000111);
			updateValue(
					NumberUtil.readDouble(bytes, commandClassIndex + 4, size, precision));
			break;
		}

	}

	@Override
	public String toString() {
		return "m:" + getModeId() + ",v=" + getLastKnownValue();
	}
}
