/*
 * Java
 *
 * Copyright 2014-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.basedriver.zwave.frame;

import java.io.IOException;

import ej.basedriver.Controller;
import ej.basedriver.zwave.Session;
import ej.basedriver.zwave.ZwaveController;
import ej.basedriver.zwave.ZwaveNode;
import ej.basedriver.zwave.ZwaveProtocol;

public class AddNodeToNetwork extends RequestFrame {

	public static RequestFrame inclusionStart (RequestFrameListener listener) {
		AddNodeToNetwork frame = new AddNodeToNetwork(listener, Session.INCLUSION);
		frame.addParameter(ZwaveProtocol.ADD_NODE_PARAMETERS);
		frame.addParameter((byte) 0xFF);
		return frame;
	}

	public static RequestFrame inclusionStop(RequestFrameListener listener) {
		AddNodeToNetwork frame = new AddNodeToNetwork(listener, Session.NONE);
		frame.addParameter(ZwaveProtocol.ADD_NODE_STOP);
		return frame;
	}

	private final Session session;

	/**
	 * @param listener
	 * @param functionID
	 */
	private AddNodeToNetwork(RequestFrameListener listener, Session session) {
		super(listener, ZwaveProtocol.FUNC_ID_ZW_ADD_NODE_TO_NETWORK, 2);
		this.session = session;
	}

	@Override
	public Session getSession() {
		return session;
	}

	@Override
	public void handleRequest(ZwaveController controller, DataFrame dataFrame)
			throws IOException {
		byte [] bytes =  dataFrame.getBytes();
		byte status = bytes[5];
		switch (status) {
		case ZwaveProtocol.ADD_NODE_STATUS_LEARN_READY:
			Session.LEARN_READY.close();
			controller.setState(Controller.INCLUSION);
			break;
		case ZwaveProtocol.ADD_NODE_STATUS_NODE_FOUND:
			break;
		case ZwaveProtocol.ADD_NODE_STATUS_ADDING_CONTROLLER:
			Session.NODE_FOUND.close();
			createNewNode(controller, dataFrame);
			break;
		case ZwaveProtocol.ADD_NODE_STATUS_ADDING_SLAVE:
			Session.NODE_FOUND.close();
			createNewNode(controller, dataFrame);
			break;
		case ZwaveProtocol.ADD_NODE_STATUS_PROTOCOL_DONE:
			Session.ADD_NODE.close();
			break;
		case ZwaveProtocol.ADD_NODE_STATUS_FAILED:
			Session.ADD_NODE.close();
			break;
		case ZwaveProtocol.ADD_NODE_STATUS_DONE:
			Session.CALLBACK.close();
			break;
		}
	}

	/**
	 * @param nodeId
	 * @throws IOException
	 */
	private void createNewNode (ZwaveController controller, DataFrame dataFrame) throws IOException {
		byte [] bytes = dataFrame.getBytes();
		if (controller.getNode(bytes[6]) == null) {
			ZwaveNode node = new ZwaveNode(controller, bytes[6]);
			controller.sendFrame(new NodeProtocolInfo(controller, node));
			int len = (bytes[LENGTH_INDEX] & 0xFF) - 3; /* 3 for basic,generic,specific*/
			if (len > 0) {
				int cpt = 0;
				boolean foundMark = false;
				while(!foundMark && (cpt < len)) {
					byte commandClass = bytes[FIRST_COMMAND_CLASS_INDEX + cpt];
					switch (commandClass) {
					case ZwaveProtocol.COMMAND_CLASS_ASSOCIATION:
						controller.sendFrame(new SendData(node, ZwaveProtocol.COMMAND_CLASS_ASSOCIATION, ZwaveProtocol.ASSOCIATION_GROUPINGS_GET));
						break;
					case ZwaveProtocol.COMMAND_CLASS_WAKE_UP:
						controller.sendFrame(new SendData(node, ZwaveProtocol.COMMAND_CLASS_WAKE_UP, ZwaveProtocol.WAKE_UP_INTERVAL_GET));
						break;
					case ZwaveProtocol.COMMAND_CLASS_MARK:
						foundMark = true;
						break;
					default:
						break;
					}
					cpt++;
				}
			}
		}
	}

	private static final int LENGTH_INDEX                     = 7;
	private static final int FIRST_COMMAND_CLASS_INDEX        = 11;
}
