/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.data;

import java.io.IOException;

/**
 * A {@link DataReader} provides APIs for reading a structured data in streaming.
 * <p>
 * A structured data is a value that can be:
 * <ul>
 * <li>a base type: boolean, int, long, double, String.</li>
 * <li>a map: a sequence of key/value pairs. Map keys can be integer keys or String keys.</li>
 * <li>an array: a sequence of values.</li>
 * </ul>
 * <p>
 * Here are some examples of usage. The data expected to be read is displayed in the JSON format.
 * <p>
 * <ul>
 * <li>
 * Reading a base type value of type boolean.
 * <p>
 * Data:
 * <pre>
 * 	true
 * </pre>
 * Code:
 * <pre>
 * 	dr.readBooleanValue(); // true
 * </pre>
 * </li>
 * <li>
 * Reading an array:
 * <p>
 * Data:
 * <pre>
 * 	[ true, "MicroEJ", 2016 ]
 * </pre>
 * Code:
 * <pre>
 * 	dr.readArrayValueStart();
 * 	dr.readBooleanValue();  // true
 * 	dr.readStringValue();   // "MicroEJ"
 * 	dr.readIntValue();      // 2016
 * 	dr.readArrayValueEnd();
 * </pre>
 * </li>
 * <li>
 * Reading a map:
 * <p>
 * Data:
 * <pre>
 * 	{ "aKey1" : "MicroEJ", "aKey2" : 2016 }
 * </pre>
 * Code:
 * <pre>
 * 	dr.readMapValueStart();
 * 	dr.readKeyString();  // "aKey1"
 * 	dr.readStringValue();  // "MicroEJ"
 * 	dr.readKeyString();   // "aKey2"
 * 	dr.readIntValue(); // 2016
 * 	dr.readMapValueEnd();
 * </pre>
 * </li>
 * </ul>
 */
public interface DataReader {

	/**
	 * Reads the next expected boolean value.
	 * @return the boolean value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	boolean readBooleanValue() throws IOException;

	/**
	 * Reads the next expected integer value (signed 32bits).
	 * @return the int value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	int readIntValue() throws IOException;

	/**
	 * Reads the next expected long integer value (signed 64bits).
	 * @return the long value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	long readLongValue() throws IOException;

	/**
	 * Reads the next expected double precision floating point value (64bits).
	 * @return the double value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	double readDoubleValue() throws IOException;

	/**
	 * Reads the next expected {@link String} value.
	 * @return the String value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	String readStringValue() throws IOException;

	/**
	 * Reads the next expected integer key (signed 32bits) within the current map.
	 * @return the int key.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the current collection is not a map or if a map value is expected.
	 */
	int readIntKey() throws IOException;

	/**
	 * Reads the next expected {@link String} key within the current map.
	 * @return the String key.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the current collection is not a map or if a map value is expected.
	 */
	String readStringKey() throws IOException;

	/**
	 * Reads the beginning of an array value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	void readArrayValueStart() throws IOException;

	/**
	 * Reads the end of an array value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the current collection is not an array.
	 */
	void readArrayValueEnd() throws IOException;

	/**
	 * Reads the beginning of a map value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	void readMapValueStart() throws IOException;

	/**
	 * Reads the end of a map value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the current collection is not a map of if a key has been previously written without a value.
	 */
	void readMapValueEnd() throws IOException;

	/**
	 * Tells whether the current collection has more elements.
	 * @return true if there are elements to be read, false if the end of the current collection or the end of the data structure is reached.
	 * @throws IOException on I/O error.
	 */
	boolean hasMoreElements() throws IOException;

	/**
	 * Skip the next value.
	 * @throws IOException on I/O error.
	 * @throws IllegalStateException if the end of the data is reached or if a map key is expected.
	 */
	void skipValue() throws IOException;
}
