/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector;

import ej.annotation.Nullable;
import ej.style.Element;
import ej.style.Selector;

/**
 * A type name selector selects by checking the element type name (Java class name).
 * <p>
 * Equivalent to <code>element</code> selector in CSS. Its specificity is (0,0,0,1).
 *
 * @see Object#getClass()
 * @see Class#getSimpleName()
 * @see SpecificityHelper
 */
public class TypeNameSelector implements Selector {

	private final String typeName;

	/**
	 * Creates a type name selector.
	 * <p>
	 * Equivalent to {@link #TypeNameSelector(String)} passing the type simple name.
	 *
	 * @param type
	 *            the type to check.
	 *
	 * @see Class#getSimpleName()
	 */
	public TypeNameSelector(Class<?> type) {
		this(type.getSimpleName());
	}

	/**
	 * Creates a TypeNameSelector.
	 * <p>
	 * Note that the case of the type name is ignored.
	 *
	 * @param typeName
	 *            the name of the type to check.
	 */
	public TypeNameSelector(String typeName) {
		this.typeName = typeName.toLowerCase();
	}

	@Override
	public boolean fit(Element element) {
		return this.typeName.equals(element.getClass().getSimpleName().toLowerCase());
	}

	@Override
	public int getSpecificity() {
		return SpecificityHelper.getSpecificity(0, 0, 0, 1);
	}

	@Override
	public int hashCode() {
		return this.typeName.hashCode();
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj instanceof TypeNameSelector) {
			return ((TypeNameSelector) obj).typeName == this.typeName;
		}
		return false;
	}

}
