/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.selector.combinator;

import java.util.Arrays;

import ej.annotation.Nullable;
import ej.style.Selector;

/**
 * A combinator is a selector that aggregates other selectors.
 * <p>
 * The specificity of a combinator is the sum of the specificities of its children.
 */
public abstract class Combinator implements Selector {

	/**
	 * Wrapped selectors list.
	 */
	protected final Selector[] selectors;
	private final int specificity;

	/**
	 * Creates a combinator with two children.
	 *
	 * @param selector1
	 *            the first child.
	 * @param selector2
	 *            the second child.
	 */
	public Combinator(Selector selector1, Selector selector2) {
		this.selectors = new Selector[2];
		this.selectors[0] = selector1;
		this.selectors[1] = selector2;
		this.specificity = selector1.getSpecificity() + selector2.getSpecificity();
	}

	/**
	 * Creates a combinator with several children.
	 *
	 * @param selectors
	 *            the list of selectors.
	 * @throws IllegalArgumentException
	 *             if there is less than two selectors.
	 */
	public Combinator(Selector... selectors) {
		int length = selectors.length;
		if (length < 2) {
			throw new IllegalArgumentException();
		}
		this.selectors = new Selector[length];
		System.arraycopy(selectors, 0, this.selectors, 0, length);
		int total = 0;
		for (Selector selector : selectors) {
			total += selector.getSpecificity();
		}
		this.specificity = total;
	}

	@Override
	public int getSpecificity() {
		return this.specificity;
	}

	@Override
	public int hashCode() {
		return this.selectors.hashCode();
	}

	@Override
	public boolean equals(@Nullable Object obj) {
		if (obj != null && obj.getClass() == getClass()) {
			return Arrays.equals(((Combinator) obj).selectors, this.selectors);
		}
		return false;
	}

}