/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.style.util;

import java.util.ArrayList;
import java.util.List;

import ej.annotation.NonNull;
import ej.annotation.Nullable;
import ej.style.Element;
import ej.style.State;
import ej.style.Style;

/**
 * Convenient and simple implementation of an element.
 */
public class ElementAdapter implements Element {

	private final List<String> classSelectors;
	@Nullable
	private final Element parent;
	@Nullable
	private Style style;

	/**
	 * Creates an element adapter without parent.
	 */
	public ElementAdapter() {
		this.parent = null;
		this.classSelectors = new ArrayList<>(0);
	}

	/**
	 * Creates an element adapter.
	 *
	 * @param parent
	 *            the element parent.
	 */
	public ElementAdapter(Element parent) {
		this.parent = parent;
		this.classSelectors = new ArrayList<>(0);
	}

	/**
	 * Gets the current style of the element.
	 * <p>
	 * Must be called in display context to avoid concurrent access to stylesheet.
	 *
	 * @return the current style of the element.
	 */
	@NonNull
	public Style getStyle() {
		Style style = this.style;
		if (style == null) {
			style = StyleHelper.getStylesheet().getStyle(this);
			this.style = style;
		}
		return style;
	}

	/**
	 * Updates the style of the element.
	 * <p>
	 * Must be called in display context to avoid concurrent access to stylesheet.
	 */
	public void updateStyle() {
		this.style = StyleHelper.getStylesheet().getStyle(this);
	}

	/**
	 * Adds a class selector.
	 *
	 * @param classSelector
	 *            the class selector to add.
	 * @throws IllegalArgumentException
	 *             if the given string contains one or more space.
	 * @throws NullPointerException
	 *             if the given class selector is <code>null</code>.
	 */
	public void addClassSelector(@NonNull String classSelector) {
		if (classSelector.indexOf(' ') != -1) {
			throw new IllegalArgumentException();
		}
		this.classSelectors.add(classSelector);
		updateStyle();
	}

	/**
	 * Removes a class selector.
	 *
	 * @param classSelector
	 *            the class selector to remove.
	 */
	public void removeClassSelector(@NonNull String classSelector) {
		this.classSelectors.remove(classSelector);
		updateStyle();
	}

	@Override
	public boolean hasClassSelector(@NonNull String classSelector) {
		return this.classSelectors.contains(classSelector);
	}

	/**
	 * Gets an array containing all the class selectors of this element.
	 *
	 * @return the class selectors.
	 */
	@NonNull
	public String[] getClassSelectors() {
		return this.classSelectors.toArray(new String[this.classSelectors.size()]);
	}

	@Override
	public boolean isInState(State state) {
		return false;
	}

	@Override
	@Nullable
	public String getAttribute(String attribute) {
		return null;
	}

	@Override
	@Nullable
	public Element getParentElement() {
		return this.parent;
	}

	@Override
	@NonNull
	public Element[] getChildrenElements() {
		return EMPTY_ELEMENT_ARRAY;
	}

	@Override
	@NonNull
	public Element getChild(int index) {
		throw new ArrayIndexOutOfBoundsException();
	}

	@Override
	public int getChildrenCount() {
		return 0;
	}

}
