/*
 * Java
 *
 * Copyright 2015-2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.application;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * An application represents a deployment unit.
 * <p>
 * Two applications are equals if and only if they have the same identifier.
 */
public interface Application {

	/**
	 * Gets the application unique identifier.
	 * <p>
	 * The identifier remains the same even between different application version.
	 * <p>
	 * This identifier is a unique identifier in the application ecosystem (for example an application store) and may
	 * not be the same as the one in the application MANIFEST.MF.
	 *
	 * @return the application identifier.
	 */
	String getIdentifier();

	/**
	 * Gets the application version.
	 * <p>
	 * This version depends on the application ecosystem (for example an application store) and may not be the same as
	 * the one in the application MANIFEST.MF.
	 *
	 * @return the application version.
	 */
	String getVersion();

	/**
	 * Gets whether the application is started or not.
	 *
	 * @return <code>true</code> if the application is started, <code>false</code> otherwise.
	 */
	boolean isStarted();

	/**
	 * Causes this application to start. A new thread owned by the application is created and started. The application
	 * became started and this method returns.
	 *
	 * @throws IllegalStateException
	 *             if the application state is already started.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while starting this application.
	 * @see #isStarted()
	 */
	void start() throws IllegalStateException, SecurityException, ApplicationOperationException;

	/**
	 * Causes this application to start. A new thread owned by the application is created and started. The application
	 * became started and this method returns.
	 *
	 * @param monitor
	 *            the progress monitor to use for reporting progress to the user. It is the caller's responsibility to
	 *            call done() on the given monitor.
	 * @throws IllegalStateException
	 *             if the application state is already started.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while starting this application.
	 * @see #isStarted()
	 */
	void start(IProgressMonitor monitor) throws IllegalStateException, SecurityException, ApplicationOperationException;

	/**
	 * Causes this application to stop. The application became stopped and this method returns.
	 *
	 * @throws IllegalStateException
	 *             if the application is not started.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while stopping this application.
	 * @see #isStarted()
	 */
	void stop() throws IllegalStateException, SecurityException, ApplicationOperationException;

	/**
	 * Causes this application to stop. The application became stopped and this method returns.
	 *
	 * @param monitor
	 *            the progress monitor to use for reporting progress to the user. It is the caller's responsibility to
	 *            call done() on the given monitor.
	 * @throws IllegalStateException
	 *             if the application is not started.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while stopping this application.
	 * @see #isStarted()
	 */
	void stop(IProgressMonitor monitor) throws IllegalStateException, SecurityException, ApplicationOperationException;

}
