/*
 * Java
 *
 * Copyright 2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.application;

import java.io.InputStream;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * The application installer allows to install or uninstall applications.
 */
public interface ApplicationInstaller {

	/**
	 * Installs an application from an input stream.
	 *
	 * @param stream
	 *            the stream containing the application.
	 * @return the application instance.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while installing this application.
	 */
	Application install(InputStream stream) throws ApplicationOperationException;

	/**
	 * Installs an application from an input stream.
	 *
	 * @param stream
	 *            the stream containing the application.
	 * @param monitor
	 *            the progress monitor to use for reporting progress to the user. It is the caller's responsibility to
	 *            call done() on the given monitor.
	 * @return the application instance.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while installing this application.
	 */
	Application install(InputStream stream, IProgressMonitor monitor) throws ApplicationOperationException;

	/**
	 * Uninstalls an application.
	 *
	 * @param application
	 *            the application to uninstall.
	 * @return {@code true} if the application has been uninstalled, {@code false} otherwise.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while uninstalling this application.
	 */
	boolean uninstall(Application application) throws ApplicationOperationException;

	/**
	 * Uninstalls an application.
	 *
	 * @param application
	 *            the application to uninstall.
	 * @param monitor
	 *            the progress monitor to use for reporting progress to the user. It is the caller's responsibility to
	 *            call done() on the given monitor.
	 * @return {@code true} if the application has been uninstalled, {@code false} otherwise.
	 * @throws SecurityException
	 *             if a security manager exits and if the caller does not have the appropriate permissions.
	 * @throws ApplicationOperationException
	 *             if any unexpected errors occur while uninstalling this application.
	 */
	boolean uninstall(Application application, IProgressMonitor monitor) throws ApplicationOperationException;

	/**
	 * Adds an application installation listener.
	 * <p>
	 * It will be notified for each application installation or uninstallation.
	 *
	 * @param applicationInstallationListener
	 *            the listener to add.
	 */
	void addApplicationInstallationListener(ApplicationInstallationListener applicationInstallationListener);

	/**
	 * Removes an application installation listener.
	 * <p>
	 * If the given listener is not registered, nothing is done.
	 *
	 * @param applicationInstallationListener
	 *            the listener to remove.
	 */
	void removeApplicationInstallationListener(ApplicationInstallationListener applicationInstallationListener);

}
