/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.wadapps.management;

import ej.observable.Observable;
import ej.observable.Observer;
import ej.wadapps.app.Notification;

/**
 * A notification manager holds incoming notifications until they are cancelled.
 */
public interface NotificationsManager {

	/**
	 * Adds an observer to this notifications manager.
	 * <p>
	 * The observer will be notified when notifications will be posted or canceled.
	 *
	 * @param observer
	 *            an observer to be added.
	 * @throws NullPointerException
	 *             if the given observer is <code>null</code>.
	 * @see Observable#addObserver(Observer)
	 * @see #notify(Notification)
	 * @see #cancel(String)
	 * @see #cancelAll()
	 */
	void addObserver(Observer observer) throws NullPointerException;

	/**
	 * Deletes an observer.
	 * <p>
	 * The observer will no longer be notified of notifications events.
	 *
	 * @param observer
	 *            an observer to be removed.
	 * @see Observable#deleteObserver(Observer)
	 */
	void deleteObserver(Observer observer);

	/**
	 * Posts a notification.
	 * <p>
	 * The observers will be notified with the notification tag as argument.
	 *
	 * @param notification
	 *            the notification to be posted.
	 */
	void notify(Notification notification);

	/**
	 * Cancels a notification.
	 * <p>
	 * The observers will be notified with the tag as argument.
	 *
	 * @param tag
	 *            the tag to cancel.
	 */
	void cancel(String tag);

	/**
	 * Cancels all notifications.
	 * <p>
	 * The observers will be notified with a <code>null</code> argument.
	 */
	void cancelAll();

	/**
	 * Gets all active notifications.
	 *
	 * @return the notifications.
	 */
	Notification[] getNotifications();

}
