/*
 * Java
 *
 * Copyright 2018-2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.monosandbox.application;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import ej.util.message.Level;
import ej.wadapps.application.Application;
import ej.wadapps.application.ApplicationOperationException;
import ej.wadapps.application.ApplicationPermission;
import ej.wadapps.monosandbox.util.WadappsMonoSandboxMessage;

/**
 * Application stub without sandboxing.
 */
public abstract class AbstractApplicationMonoSandbox implements Application {

	private static final String START = "Start"; //$NON-NLS-1$
	private static final String STOP = "Stop"; //$NON-NLS-1$

	private final AbstractApplicationManagerMonoSandbox applicationManagerMonoSandbox;

	private boolean started;

	/**
	 * Creates an application without sandboxing.
	 *
	 * @param applicationManager
	 *            the application manager instance.
	 */
	public AbstractApplicationMonoSandbox(AbstractApplicationManagerMonoSandbox applicationManager) {
		this.applicationManagerMonoSandbox = applicationManager;
	}

	@Override
	public boolean isStarted() {
		return this.started;
	}

	/**
	 * Creates the entry point associated with the application.
	 * <p>
	 * The reference to the entry point must be kept to be started and stopped later.
	 */
	protected abstract void createEntryPoint();

	/**
	 * Starts the entry point associated with the application.
	 */
	protected abstract void startEntryPoint();

	/**
	 * Stops the entry point associated with the application.
	 */
	protected abstract void stopEntryPoint();

	@Override
	public void start() throws IllegalStateException, SecurityException, ApplicationOperationException {
		start(new NullProgressMonitor());
	}

	@Override
	public synchronized void start(IProgressMonitor monitor)
			throws IllegalStateException, SecurityException, ApplicationOperationException {
		if (this.started) {
			throw new IllegalStateException();
		}
		checkPermission(this, ApplicationPermission.START_ACTION);
		monitor.beginTask(START, 3);
		try {
			createEntryPoint();
		} catch (Throwable e) {
			WadappsMonoSandboxMessage.LOGGER.log(Level.INFO, WadappsMonoSandboxMessage.CATEGORY,
					WadappsMonoSandboxMessage.USER_ERROR, e);
		}
		monitor.worked(1);
		new Thread() {
			@Override
			public void run() {
				try {
					startEntryPoint();
				} catch (Throwable e) {
					WadappsMonoSandboxMessage.LOGGER.log(Level.INFO, WadappsMonoSandboxMessage.CATEGORY,
							WadappsMonoSandboxMessage.USER_ERROR, e);
				}
			}
		}.start();
		monitor.worked(1);
		AbstractApplicationMonoSandbox.this.started = true;
		AbstractApplicationMonoSandbox.this.applicationManagerMonoSandbox.onStart(AbstractApplicationMonoSandbox.this);
		monitor.worked(1);
	}

	@Override
	public void stop() throws IllegalStateException, SecurityException, ApplicationOperationException {
		stop(new NullProgressMonitor());
	}

	@Override
	public synchronized void stop(IProgressMonitor monitor)
			throws IllegalStateException, SecurityException, ApplicationOperationException {
		if (!this.started) {
			throw new IllegalStateException();
		}
		checkPermission(this, ApplicationPermission.STOP_ACTION);
		monitor.beginTask(STOP, 2);
		try {
			stopEntryPoint();
		} catch (Throwable e) {
			WadappsMonoSandboxMessage.LOGGER.log(Level.INFO, WadappsMonoSandboxMessage.CATEGORY,
					WadappsMonoSandboxMessage.USER_ERROR, e);
		}
		this.started = false;
		monitor.worked(1);
		this.applicationManagerMonoSandbox.onStop(this);
		monitor.worked(1);
	}

	/**
	 * Checks that an action on a application is permitted.
	 *
	 * @param application
	 *            the application.
	 * @param action
	 *            the action.
	 */
	private void checkPermission(Application application, String action) {
		SecurityManager securityManager = System.getSecurityManager();
		if (securityManager != null) {
			securityManager.checkPermission(new ApplicationPermission(application, action));
		}
	}

}
