/*
 * Java
 *
 * Copyright 2015-2019 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */
package ej.wadapps.multisandbox.application;

import com.microej.kf.util.RunnableWithResult;

import ej.annotation.NonNull;
import ej.annotation.Nullable;
import ej.basictool.ArrayTools;
import ej.basictool.map.PackedMap;
import ej.kf.Feature;
import ej.kf.Feature.State;
import ej.kf.FeatureStateListener;
import ej.kf.Kernel;
import ej.kf.Module;
import ej.util.message.Level;
import ej.wadapps.application.Application;
import ej.wadapps.application.ApplicationLifecycleListener;
import ej.wadapps.application.ApplicationManager;
import ej.wadapps.multisandbox.util.WadappsMultiSandboxMessage;

/**
 * Used in a KF environment with perfect matching with KF.
 * <ol>
 * <li>Start: the feature is started by KF.</li>
 * <li>Stop: the feature is stopped by KF.</li>
 * </ol>
 */
public class ApplicationManagerKF implements ApplicationManager, FeatureStateListener {

	private final PackedMap<Feature, FeatureApplication> apps;
	private ApplicationLifecycleListener[] listeners;

	/**
	 * Creates an application manager KF with all the installed features.
	 *
	 * @see Kernel#getAllLoadedFeatures()
	 * @see #ApplicationManagerKF(Feature[])
	 */
	public ApplicationManagerKF() {
		this(Kernel.getAllLoadedFeatures());
	}

	/**
	 * Creates an application manager KF with the given features.
	 * <p>
	 * For each feature, an application will be created and available in {@link #getApplications()}.
	 *
	 * @param features
	 *            the feature to manage.
	 */
	public ApplicationManagerKF(Feature[] features) {
		this.apps = new PackedMap<>();
		for (Feature feature : features) {
			if (feature != null) {
				this.apps.put(feature, createApplication(feature));
			}
		}
		this.listeners = new ApplicationLifecycleListener[0];
		Kernel.addFeatureStateListener(this);
	}

	/**
	 * Creates an application from the given feature.
	 *
	 * @param feature
	 *            the feature to manage
	 * @return the application
	 */
	protected FeatureApplication createApplication(Feature feature) {
		return new FeatureApplication(feature);
	}

	/**
	 * Gets the application associated with a feature.
	 *
	 * @param feature
	 *            the feature.
	 * @return the application, or {@code null} if no application matches the given feature.
	 */
	@Nullable
	public Application getApplication(Feature feature) {
		synchronized (this.apps) {
			return this.apps.get(feature);
		}
	}

	/**
	 * Gets the feature associated with the application.
	 *
	 * @param application
	 *            the application.
	 * @return the feature, or {@code null} if no feature is associated to the given application.
	 */
	@Nullable
	public Feature getFeature(Application application) {
		if (application instanceof FeatureApplication) {
			return ((FeatureApplication) application).getFeature();
		}
		return null;
	}

	/**
	 * Adds an application associated with a feature.
	 * <p>
	 * If the feature is already managed by the application manager, nothing changes.
	 *
	 * @param feature
	 *            the feature.
	 * @return the newly created application.
	 */
	public Application addApplication(Feature feature) {
		synchronized (this.apps) {
			FeatureApplication featureApplication = this.apps.get(feature);
			if (featureApplication != null) {
				return featureApplication;
			}
			FeatureApplication value = createApplication(feature);
			this.apps.put(feature, value);
			return value;
		}
	}

	/**
	 * Removes the application associated with a feature.
	 * <p>
	 * If the feature is not managed by the application manager, nothing changes and {@code null} is returned.
	 *
	 * @param feature
	 *            the feature.
	 * @return the application associated to the given feature or {@code null} if not found.
	 */
	@Nullable
	public Application removeApplication(Feature feature) {
		synchronized (this.apps) {
			return this.apps.remove(feature);
		}
	}

	/**
	 * Gets whether a feature is manager by the application manager or not.
	 *
	 * @param feature
	 *            the feature.
	 * @return {@code true} if the feature is managed by this application manager, {@code false} otherwise.
	 */
	public boolean contains(Feature feature) {
		synchronized (this.apps) {
			return this.apps.containsKey(feature);
		}
	}

	/**
	 * Gets all available features known by this manager.
	 * <p>
	 * If there is none, this method returns an empty array.
	 *
	 * @return all available feature.
	 */
	public Feature[] getFeatures() {
		synchronized (this.apps) {
			Feature[] result = new Feature[this.apps.size()];
			this.apps.keySet().toArray(result);
			return result;
		}
	}

	/**
	 * Gets the application for a module.
	 * <p>
	 * If the given module is the kernel, the method returns {@code null}.
	 *
	 * @param module
	 *            the module.
	 * @return the application matching the module or {@code null}.
	 * @see #getApplication(Feature)
	 */
	@Nullable
	public Application getApplication(Module module) {
		if (module == null || module == Kernel.getInstance()) {
			return null;
		}
		return getApplication((Feature) module);
	}

	@Override
	public Application[] getApplications() {
		Module callerOwner = Kernel.getContextOwner();
		Kernel.enter();
		synchronized (this.apps) {
			RunnableWithResult<Application[]> runnable = new RunnableWithResult<Application[]>() {
				@Override
				protected Application[] runWithResult() {
					return new Application[ApplicationManagerKF.this.apps.size()];
				}
			};
			Kernel.runUnderContext(callerOwner, runnable);
			@SuppressWarnings("null") // See runnable above.
			@NonNull
			Application[] result = runnable.getResult();
			this.apps.values().toArray(result);
			return result;
		}
		// Kernel.exit();
	}

	@Override
	public synchronized void addApplicationLifecycleListener(final ApplicationLifecycleListener appLifecycleListener) {
		removeApplicationLifecycleListener(appLifecycleListener);
		Kernel.enter();
		this.listeners = ArrayTools.add(this.listeners, appLifecycleListener);
		// Kernel.exit();
	}

	@Override
	public synchronized void removeApplicationLifecycleListener(ApplicationLifecycleListener appLifecycleListener) {
		Kernel.enter();
		// Iterative process and test with == (do not call equals() on 2 objects owned by 2 different features).
		this.listeners = ArrayTools.remove(this.listeners, appLifecycleListener);
		// Kernel.exit();
	}

	@Override
	public void stateChanged(@Nullable Feature feature, @Nullable State previousState) {
		if (feature != null) {
			Application application = getApplication(feature);
			if (application != null) {
				switch (feature.getState()) {
				case STOPPED:
					cleanListeners(feature);
					break;
				case STARTED:
					notifyStart(application);
					break;
				case INSTALLED: // Feature stopped and clean.
					notifyStop(application);
					break;
				default:// Nothing to do.
				}
			}
		}
	}

	private void cleanListeners(Feature feature) {
		int length = this.listeners.length;
		for (int i = 0; i < length; i++) {
			ApplicationLifecycleListener listener = this.listeners[i];
			if (Kernel.getOwner(listener) == feature) {
				this.listeners = ArrayTools.remove(this.listeners, i);
			}
		}
	}

	private void notifyStart(Application application) {
		for (ApplicationLifecycleListener listener : this.listeners) {
			try {
				listener.onStart(application);
			} catch (Throwable e) {
				WadappsMultiSandboxMessage.LOGGER.log(Level.INFO, WadappsMultiSandboxMessage.CATEGORY,
						WadappsMultiSandboxMessage.USER_ERROR, e);
			}
		}
	}

	private void notifyStop(Application application) {
		for (ApplicationLifecycleListener listener : this.listeners) {
			try {
				listener.onStop(application);
			} catch (Throwable e) {
				WadappsMultiSandboxMessage.LOGGER.log(Level.INFO, WadappsMultiSandboxMessage.CATEGORY,
						WadappsMultiSandboxMessage.USER_ERROR, e);
			}
		}
	}

}
