/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget;

import ej.annotation.Nullable;
import ej.basictool.ArrayTools;
import ej.microui.display.Display;
import ej.microui.display.GraphicsContext;
import ej.mwt.Desktop;
import ej.mwt.Panel;
import ej.style.Element;
import ej.style.State;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.style.selector.InstanceSelector;
import ej.style.util.StyleHelper;

/**
 * A desktop using a style specified in a style sheet.
 */
public class StyledDesktop extends Desktop implements Element, StyledRenderable {

	// Some (most) of the code in this class is duplicated in all StyledRenderable implementors for footprint
	// optimization (flash & RAM).

	/**
	 * The cached style.
	 */
	@Nullable
	private Style style;
	/**
	 * The class selectors.
	 */
	private String[] classSelectors;

	/**
	 * Creates a styled desktop on the default display.
	 */
	public StyledDesktop() {
		this(Display.getDefaultDisplay());
	}

	/**
	 * Creates a styled desktop on a display.
	 *
	 * @param display
	 *            the display.
	 * @throws NullPointerException
	 *             if the given display is <code>null</code>.
	 */
	public StyledDesktop(Display display) {
		super(display);
		this.classSelectors = StyledHelper.EMPTY_STRING_ARRAY;
	}

	@Override
	@Nullable
	public Element getParentElement() {
		return null;
	}

	@Override
	public Element[] getChildrenElements() {
		Panel[] panels = getPanels();
		int panelsLength = panels.length;
		Element[] result = new Element[panelsLength];
		System.arraycopy(panels, 0, result, 0, panelsLength);
		return result;
	}

	@Override
	public Element getChild(int index) {
		return (Element) getPanels()[index];
	}

	@Override
	public int getChildrenCount() {
		return getPanels().length;
	}

	@Override
	public void mergeStyle(Style style) {
		StyleHelper.getStylesheet().addRule(new InstanceSelector(this), style);
		updateStyle();
	}

	@Override
	public void removeInstanceStyle() {
		StyleHelper.getStylesheet().removeRule(new InstanceSelector(this));
		updateStyle();
	}

	@Override
	public Style getStyle() {
		Style style = this.style;
		if (style == null) {
			style = StyledHelper.getStyleFromStylesheet(this);
			setStyle(style);
		}
		return style;
	}

	private void setStyle(Style newStyle) {
		this.style = newStyle;
	}

	/**
	 * Updates the style of the widget.
	 */
	protected void updateStyle() {
		// Do not update style if not shown. See validate method.
		if (isShown() && updateStyleOnly()) {
			Panel[] panels = getPanels();
			for (Panel child : panels) {
				((StyledPanel) child).updateStyle();
			}
			repaint();
		}
	}

	/**
	 * Update widget style without repainting it.
	 *
	 * @return <code>true</code> if the style has been updated, <code>false</code> otherwise.
	 */
	protected boolean updateStyleOnly() {
		Style newStyle = StyledHelper.getStyleFromStylesheet(this);

		if (!newStyle.equals(this.style)) {
			setStyle(newStyle);
			return true;
		}
		return false;
	}

	@Override
	public void render(GraphicsContext g) {
		StyledHelper.render(g, this);
	}

	@Override
	public void validate() {
		updateStyleOnly();
		super.validate();
	}

	@Override
	public void renderContent(GraphicsContext g, Style style, Rectangle bounds) {
		// Do nothing by default.
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle bounds) {
		// Not used.
		return new Rectangle();
	}

	@Override
	public boolean hasClassSelector(String classSelector) {
		return ArrayTools.containsEquals(this.classSelectors, classSelector);
	}

	@Override
	public void addClassSelector(String classSelector) {
		this.classSelectors = StyledHelper.addClassSelector(this.classSelectors, classSelector);
		updateStyle();
	}

	@Override
	public void removeClassSelector(String classSelector) {
		int classSelectorsLength = this.classSelectors.length;
		this.classSelectors = ArrayTools.remove(this.classSelectors, classSelector);
		if (this.classSelectors.length != classSelectorsLength) {
			updateStyle();
		}
	}

	@Override
	public void setClassSelectors(String classSelector) {
		this.classSelectors = StyledHelper.setClassSelectors(classSelector);
		updateStyle();
	}

	@Override
	public void removeAllClassSelectors() {
		if (this.classSelectors.length != 0) {
			this.classSelectors = StyledHelper.EMPTY_STRING_ARRAY;
			updateStyle();
		}
	}

	@Override
	public boolean isInState(State state) {
		return false;
	}

	@Override
	@Nullable
	public String getAttribute(String attribute) {
		return null;
	}

}
