/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget;

import ej.microui.display.GraphicsContext;
import ej.mwt.Renderable;
import ej.style.Style;
import ej.style.Stylesheet;
import ej.style.border.Border;
import ej.style.container.Rectangle;
import ej.style.outline.Outline;
import ej.style.selector.InstanceSelector;
import ej.style.util.StyleHelper;

/**
 * Represents a renderable with style.
 *
 * @see Stylesheet
 * @see Style
 */
public interface StyledRenderable extends Renderable {

	/**
	 * Renders the content of the renderable without the border, margin and padding specified in the style.
	 * <p>
	 * The given graphics context is translated and clipped according to the given bounds (the border, margin and
	 * padding are applied on this graphics context before).
	 *
	 * @param g
	 *            the graphics context where to render the content of the renderable.
	 * @param style
	 *            the style to use.
	 * @param bounds
	 *            the remaining size to render the content.
	 * @see Outline#apply(GraphicsContext, Rectangle)
	 * @see Border#apply(GraphicsContext, Rectangle, int)
	 * @see StyleHelper#renderWithoutContent(GraphicsContext, Rectangle, Style)
	 */
	void renderContent(GraphicsContext g, Style style, Rectangle bounds);

	/***
	 * Gets the content size of the renderable without the border, margin and padding specified in the style.
	 *
	 * @param style
	 *            the style to use.
	 * @param bounds
	 *            the bounds available for the content. A width or a height equals to <code>MWT.NONE</code> means no
	 *            constraint on this dimension.
	 *
	 * @return the content size of the renderable.
	 */
	Rectangle validateContent(Style style, Rectangle bounds);

	/**
	 * Merges a style with the style of this renderable. That means that a rule is added in the global stylesheet that
	 * applies only for this renderable with the greatest specificity.
	 * <p>
	 * Equivalent to <code>StyleHelper.getStylesheet().addRule(new InstanceSelector(renderable), style);</code>
	 *
	 * @param style
	 *            the style to merge.
	 * @see #removeInstanceStyle()
	 * @see InstanceSelector
	 */
	void mergeStyle(Style style);

	/**
	 * Removes the style specific to this renderable. That means that the rule that applies only to this renderable in
	 * the global stylesheet is removed.
	 * <p>
	 * Equivalent to <code>StyleHelper.getStylesheet().removeRule(new InstanceSelector(renderable));</code>
	 *
	 * @see #mergeStyle(Style)
	 * @see InstanceSelector
	 */
	void removeInstanceStyle();

	/**
	 * Gets the current style of the renderable.
	 * <p>
	 * The style is retrieved from the global stylesheet: <code>StyleHelper.getStylesheet().getStyle(renderable);</code>
	 *
	 * @return the current style of the renderable.
	 * @see Stylesheet#getStyle(ej.style.Element)
	 */
	Style getStyle();

	/**
	 * Adds a class selector.
	 *
	 * @param classSelector
	 *            the class selector to add.
	 * @throws IllegalArgumentException
	 *             if the given string contains one or more space.
	 * @throws NullPointerException
	 *             if the given class selector is <code>null</code>.
	 */
	void addClassSelector(String classSelector);

	/**
	 * Removes a class selector.
	 *
	 * @param classSelector
	 *            the class selector to remove.
	 */
	void removeClassSelector(String classSelector);

	/**
	 * Sets the class selectors.
	 * <p>
	 * The given string is a space separated list of class selectors.
	 * <p>
	 * If there is already some class selectors, they are removed.
	 *
	 * @param classSelectors
	 *            the class selectors list to split.
	 */
	void setClassSelectors(String classSelectors);

	/**
	 * Removes all the class selectors.
	 */
	void removeAllClassSelectors();

}
