/*
 * Java
 *
 * Copyright 2017-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.animation;

import java.util.ArrayList;
import java.util.List;

import ej.annotation.Nullable;

/**
 * The animation listener registry holds a list of animation listeners that can be notified.
 *
 * @see AnimationListener
 */
public class AnimationListenerRegistry {

	private static List<AnimationListener> AnimationListeners = newEmptyArrayList();
	private static List<String> AnimationsRunning = newEmptyArrayList();

	private static <T> ArrayList<T> newEmptyArrayList() {
		return new ArrayList<>(0);
	}

	/**
	 * Registers an animation listener.
	 * <p>
	 * An animation listener can be registered twice, it will be notified twice.
	 *
	 * @param animationListener
	 *            the animation listener to add.
	 * @throws NullPointerException
	 *             if the given listener is <code>null</code>.
	 */
	public static void register(AnimationListener animationListener) {
		if (animationListener == null) {
			throw new NullPointerException();
		}
		AnimationListeners.add(animationListener);
	}

	/**
	 * Unregisters an animation listener.
	 * <p>
	 * If the listener is <code>null</code> or not registered, nothing changes.
	 *
	 * @param animationListener
	 *            the animation listener to remove.
	 */
	public static void unregister(@Nullable AnimationListener animationListener) {
		AnimationListeners.remove(animationListener);
	}

	/**
	 * Notifies all the animation listeners that an animation is started.
	 *
	 * @param id
	 *            the id of the animation.
	 * @see AnimationListener#onStartAnimation()
	 */
	public static void notifyOnStart(String id) {
		AnimationsRunning.add(id);
		for (AnimationListener animationListener : AnimationListeners) {
			animationListener.onStartAnimation();
		}
	}

	/**
	 * Notifies all the animation listeners that an animation is stopped.
	 *
	 * @param id
	 *            the id of the animation.
	 * @see AnimationListener#onStopAnimation()
	 */
	public static void notifyOnStop(String id) {
		if (AnimationsRunning.remove(id)) {
			for (AnimationListener animationListener : AnimationListeners) {
				animationListener.onStopAnimation();
			}
		}
	}

	/**
	 * Gets whether or not an animation is running.
	 *
	 * @return <code>true</code> if an animation is running, <code>false</code> otherwise.
	 */
	public static boolean isAnimationRunning() {
		return AnimationsRunning.size() != 0;
	}

}
