/*
 * Java
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.widget.basic;

import ej.giml.annotation.Element;
import ej.giml.annotation.ElementAttribute;
import ej.giml.annotation.ElementConstructor;
import ej.microui.display.Font;
import ej.microui.display.GraphicsContext;
import ej.style.Style;
import ej.style.container.Rectangle;
import ej.style.text.TextManager;
import ej.style.util.StyleHelper;
import ej.widget.StyledWidget;

/**
 * A label is a widget that displays a text.
 * <p>
 * This example shows a simple label:
 *
 * <pre>
 * Label label = new Label("Hello World");
 * </pre>
 *
 * <img src="doc-files/label-simple.png" alt="Simple label.">
 * <p>
 * This example shows a styled label:
 *
 * <pre>
 * Label styledLabel = new Label("Hello World");
 * EditableStyle labelStyle = new EditableStyle();
 * labelStyle.setForegroundColor(Colors.NAVY);
 * labelStyle.setBackgroundColor(Colors.CYAN);
 * labelStyle.setFontProfile(new FontProfile("", 23, Font.STYLE_PLAIN));
 * styledLabel.mergeStyle(labelStyle);
 * </pre>
 *
 * <img src="doc-files/label-styled.png" alt="Styled label.">
 */
@Element
public class Label extends StyledWidget {

	private String text;

	/**
	 * Creates a label with an empty text.
	 */
	public Label() {
		this(""); //$NON-NLS-1$
	}

	/**
	 * Creates a label with the given text to display.
	 *
	 * @param text
	 *            the text to display, it cannot be <code>null</code>.
	 * @throws NullPointerException
	 *             if the text is <code>null</code>.
	 */
	@ElementConstructor
	public Label(@ElementAttribute(defaultValue = "") String text) {
		super();
		setEnabled(false);
		this.text = text;
	}

	/**
	 * Sets the text to display for this label.
	 *
	 * @param text
	 *            the text to display, it cannot be <code>null</code>.
	 * @throws NullPointerException
	 *             if the text is <code>null</code>.
	 * @see #updateText(String)
	 */
	public void setText(String text) {
		updateText(text);
		repaint();
	}

	/**
	 * Updates the text to display for this label without asking for repaint.
	 *
	 * @param text
	 *            the text to display, it cannot be <code>null</code>.
	 * @throws NullPointerException
	 *             if the text is <code>null</code>.
	 * @since 2.3.0
	 */
	public void updateText(String text) {
		this.text = text;
		invalidate();
	}

	/**
	 * Gets the displayed text of this label.
	 *
	 * @return the displayed text of this label.
	 */
	public String getText() {
		return this.text;
	}

	@Override
	public void renderContent(GraphicsContext g, Style style, Rectangle bounds) {
		Font font = StyleHelper.getFont(style);
		TextManager textManager = style.getTextManager();
		textManager.drawText(g, this.text, font, style.getForegroundColor(), bounds, style.getAlignment());
	}

	@Override
	public Rectangle validateContent(Style style, Rectangle availableSize) {
		Font font = StyleHelper.getFont(style);
		TextManager textManager = style.getTextManager();
		return textManager.computeContentSize(this.text, font, availableSize);
	}
}
