/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package googlecloud.iotcore.pubsub;

import javax.net.ssl.SSLContext;

import org.json.me.JSONException;

import com.microej.signature.SignatureException;
import com.microej.signature.Signer;

import googlecloud.iotcore.pubsub.jwt.JWTBuilder;

/**
 * This class provides convenience methods to help build addresses, topics and IDs in relation to Google Cloud Platform
 * IoT Core.
 */
public class ConnectOptions {

	private String serverHost;

	private int serverPort;

	private String projectId;

	private String cloudRegion;

	private String registryId;

	private String deviceId;

	private SSLContext sslContext;

	private long jwtSessionPeriod;

	private Signer jwtSigner;

	/**
	 * Gets the serverHost.
	 *
	 * @return the serverHost.
	 */
	public String getServerHost() {
		return this.serverHost;
	}

	/**
	 * Sets the serverHost.
	 *
	 * @param serverHost
	 *            the serverHost to set.
	 * @return the current instance of the options
	 */
	public ConnectOptions setServerHost(String serverHost) {
		this.serverHost = serverHost;
		return this;
	}

	/**
	 * Gets the serverPort.
	 *
	 * @return the serverPort.
	 */
	public int getServerPort() {
		return this.serverPort;
	}

	/**
	 * Sets the serverPort.
	 *
	 * @param serverPort
	 *            the serverPort to set.
	 * @return the current instance of the options
	 */
	public ConnectOptions setServerPort(int serverPort) {
		this.serverPort = serverPort;
		return this;
	}

	/**
	 * Sets the projectId.
	 *
	 * @param projectId
	 *            the projectId to set.
	 * @return the current instance of the options
	 */
	public ConnectOptions setProjectId(String projectId) {
		this.projectId = projectId;
		return this;
	}

	/**
	 * Sets the cloudRegion.
	 *
	 * @param cloudRegion
	 *            the cloudRegion to set.
	 * @return the current instance of the options
	 */
	public ConnectOptions setCloudRegion(String cloudRegion) {
		this.cloudRegion = cloudRegion;
		return this;
	}

	/**
	 * Sets the registryId.
	 *
	 * @param registryId
	 *            the registryId to set.
	 * @return the current instance of the options
	 */
	public ConnectOptions setRegistryId(String registryId) {
		this.registryId = registryId;
		return this;
	}

	/**
	 * Sets the deviceId.
	 *
	 * @param deviceId
	 *            the deviceId to set.
	 * @return the current instance of the options
	 */
	public ConnectOptions setDeviceId(String deviceId) {
		this.deviceId = deviceId;
		return this;
	}

	/**
	 * Gets the sslContext.
	 *
	 * @return the sslContext.
	 */
	public SSLContext getSslContext() {
		return this.sslContext;
	}

	/**
	 * Sets the sslContext.
	 *
	 * @param sslContext
	 *            the sslContext to set.
	 * @return the current instance of the options
	 */
	public ConnectOptions setSslContext(SSLContext sslContext) {
		this.sslContext = sslContext;
		return this;
	}

	/**
	 *
	 * @param period
	 *            expressed in seconds
	 * @return the current instance of the options
	 */
	public ConnectOptions setJWTSessionPeriod(long period) {
		this.jwtSessionPeriod = period;
		return this;
	}

	/**
	 * Set the signer that will be used to generate the JWT.
	 *
	 * @param signer
	 *            a Signer from the signature API
	 * @return the current instance of the options
	 */
	public ConnectOptions setJWTSigner(Signer signer) {
		this.jwtSigner = signer;
		return this;
	}

	/**
	 * Build a JSON Web Token suitable for Google Cloud IoT Core communication.
	 *
	 * @return the signed data of the JWT
	 * @throws JSONException
	 *             when building the JSON part JWT is failing
	 * @throws SignatureException
	 *             when the signature has failed
	 * @throws IllegalArgumentException
	 *             when fields are not correct
	 */
	public char[] getJWT() throws JSONException, SignatureException, IllegalArgumentException {

		JWTBuilder tokenBuilder = new JWTBuilder();
		long now = System.currentTimeMillis() / 1000;
		long expirationTime = now + this.jwtSessionPeriod;
		tokenBuilder.setSigner(this.jwtSigner).setIssuedAt(now).setExpiration(expirationTime)
				.setAudience(this.projectId);

		return tokenBuilder.build().toCharArray();
	}

	/**
	 *
	 * @return the correctly formatted server address
	 */
	public String buildServerAddress() {
		return "ssl://" + this.serverHost + ":" + this.serverPort;
	}

	/**
	 * Build the specific client ID needed to connect to GCP IoT Core MQTT broker.
	 *
	 * @return a String built as follows
	 *         "projects/projectId/locations/cloudRegion/registries/registryId/devices/deviceId"
	 */
	public String buildMqttClientId() {
		return "projects/" + this.projectId + "/locations/" + this.cloudRegion + "/registries/" + this.registryId
				+ "/devices/" + this.deviceId;
	}

	/**
	 * Build a topic prefixed with the device set into this builder.
	 *
	 * @param subTopic
	 *            the topic suffix you want to use
	 * @return the real topic that should be used in the GCP IoT Core MQTT broker
	 */
	public String buildTopic(String subTopic) {
		return "/devices/" + this.deviceId + "/" + subTopic;
	}

}
