/*
 * Java
 *
 * Copyright 2018-2019 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package googlecloud.iotcore.pubsub.jwt;

import java.util.Base64;
import java.util.Base64.Encoder;

import org.json.me.JSONException;
import org.json.me.JSONObject;

import com.microej.signature.SignatureException;
import com.microej.signature.Signer;

/**
 * Build a JSON Web Token compliant with Google Cloud IoT Core services.
 */
public class JWTBuilder {

	/**
	 * Already computed base64 encoded JWT header.
	 *
	 * { "alg": "RS256", "typ": "JWT" }
	 */
	private static final String JWT_HEADER = "eyAiYWxnIjogIlJTMjU2IiwgInR5cCI6ICJKV1QiIH0NCg";

	private static final long ONE_HOUR = 3600000L;

	private Signer signer;

	private long issuedAt = 0;

	private long expiration = 0;

	private String audience = null;

	/**
	 *
	 * @param value
	 *            the signer containing the private key information
	 * @return the current instance of JWTBuilder
	 */
	public JWTBuilder setSigner(Signer value) {
		this.signer = value;
		return this;
	}

	/**
	 *
	 * @param value
	 *            the UNIX timestamp in seconds of the JWT creation
	 * @return the current instance of JWTBuilder
	 */
	public JWTBuilder setIssuedAt(long value) {
		this.issuedAt = value;
		return this;
	}

	/**
	 *
	 * @param value
	 *            the UNIX timestamp in seconds of the JWT expiration
	 * @return the current instance of JWTBuilder
	 */
	public JWTBuilder setExpiration(long value) {
		this.expiration = value;
		return this;
	}

	/**
	 *
	 * @param value
	 *            the audience to of the JWT (usually projectID)
	 * @return the current instance of JWTBuilder
	 */
	public JWTBuilder setAudience(String value) {
		this.audience = value;
		return this;
	}

	/**
	 * Build a signed JWT from the added fields
	 *
	 * If issuedAt not specified default to current time.
	 *
	 * If expiration not specified default to one hour.
	 *
	 * @return a String containing the whole JWT header, payload and signature
	 *
	 * @throws JSONException
	 *             when building the JSON part JWT is failing
	 * @throws SignatureException
	 *             when the signature has failed
	 * @throws IllegalArgumentException
	 *             when fields are not correct
	 */
	public String build() throws JSONException, SignatureException, IllegalArgumentException {

		if (this.signer == null) {
			throw new IllegalArgumentException("No signer provided.");
		}

		if (this.audience == null) {
			throw new IllegalArgumentException("No audience provided.");
		}

		if (this.issuedAt == 0) {
			this.issuedAt = System.currentTimeMillis();
		}

		if (this.expiration == 0 || this.expiration == this.issuedAt) {
			this.expiration = this.issuedAt + ONE_HOUR;
		}

		JSONObject payload = new JSONObject();

		payload.put("aud", this.audience);
		payload.put("iat", this.issuedAt);
		payload.put("exp", this.expiration);

		Encoder base64Encoder = Base64.getUrlEncoder().withoutPadding();

		byte[] base64Payload = base64Encoder.encode(payload.toString().getBytes());

		int jwtHeaderLength = JWT_HEADER.length();
		int base64PayloadLength = base64Payload.length;
		byte[] jwtBytesToSign = new byte[jwtHeaderLength + base64PayloadLength + 1];

		System.arraycopy(JWT_HEADER.getBytes(), 0, jwtBytesToSign, 0, jwtHeaderLength);
		jwtBytesToSign[jwtHeaderLength] = '.';
		System.arraycopy(base64Payload, 0, jwtBytesToSign, jwtHeaderLength + 1, base64PayloadLength);

		byte[] signature = this.signer.sign(jwtBytesToSign);

		String encodedSignatureString = base64Encoder.encodeToString(signature);

		StringBuilder sb = new StringBuilder();

		sb.append(new String(jwtBytesToSign)).append('.').append(encodedSignatureString);

		String finalJWT = sb.toString();

		return finalJWT;
	}

}
