/*
 * Copyright (c) 1997, 2013, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015-2018, IS2T. EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package java.security.cert;

import javax.security.auth.x500.X500Principal;

/**
 * <p>
 * Abstract class for X.509 certificates. This provides a standard way to access all the attributes of an X.509
 * certificate.
 * <p>
 * In June of 1996, the basic X.509 v3 format was completed by ISO/IEC and ANSI X9, which is described below in ASN.1:
 *
 * <pre>
 * Certificate  ::=  SEQUENCE  {
 *     tbsCertificate       TBSCertificate,
 *     signatureAlgorithm   AlgorithmIdentifier,
 *     signature            BIT STRING  }
 * </pre>
 * <p>
 * These certificates are widely used to support authentication and other functionality in Internet security systems.
 * Common applications include Privacy Enhanced Mail (PEM), Transport Layer Security (SSL), code signing for trusted
 * software distribution, and Secure Electronic Transactions (SET).
 * <p>
 * These certificates are managed and vouched for by <em>Certificate Authorities</em> (CAs). CAs are services which
 * create certificates by placing data in the X.509 standard format and then digitally signing that data. CAs act as
 * trusted third parties, making introductions between principals who have no direct knowledge of each other. CA
 * certificates are either signed by themselves, or by some other CA such as a "root" CA.
 * <p>
 * More information can be found in <a href="http://www.ietf.org/rfc/rfc3280.txt">RFC 3280: Internet X.509 Public Key
 * Infrastructure Certificate and CRL Profile</a>.
 * <p>
 * The ASN.1 definition of {@code tbsCertificate} is:
 *
 * <pre>
 * TBSCertificate  ::=  SEQUENCE  {
 *     version         [0]  EXPLICIT Version DEFAULT v1,
 *     serialNumber         CertificateSerialNumber,
 *     signature            AlgorithmIdentifier,
 *     issuer               Name,
 *     validity             Validity,
 *     subject              Name,
 *     subjectPublicKeyInfo SubjectPublicKeyInfo,
 *     issuerUniqueID  [1]  IMPLICIT UniqueIdentifier OPTIONAL,
 *                          -- If present, version must be v2 or v3
 *     subjectUniqueID [2]  IMPLICIT UniqueIdentifier OPTIONAL,
 *                          -- If present, version must be v2 or v3
 *     extensions      [3]  EXPLICIT Extensions OPTIONAL
 *                          -- If present, version must be v3
 *     }
 * </pre>
 * <p>
 * Certificates are instantiated using a certificate factory. The following is an example of how to instantiate an X.509
 * certificate:
 *
 * <pre>
 * try (InputStream inStream = new FileInputStream("fileName-of-cert")) {
 * 	CertificateFactory cf = CertificateFactory.getInstance("X.509");
 * 	X509Certificate cert = (X509Certificate) cf.generateCertificate(inStream);
 * }
 * </pre>
 *
 * @author Hemma Prafullchandra
 *
 *
 * @see Certificate
 * @see CertificateFactory
 * @see X509Extension
 */

public abstract class X509Certificate extends Certificate implements X509Extension {

	private static final long serialVersionUID = -2491127588187038216L;

	/**
	 * Constructor for X.509 certificates.
	 */
	protected X509Certificate() {
		super("X.509");
	}

	/**
	 * Returns the issuer (issuer distinguished name) value from the certificate as an <code>X500Principal</code>.
	 * <p>
	 * It is recommended that subclasses override this method.
	 *
	 * @return an <code>X500Principal</code> representing the issuer distinguished name
	 * @since 1.4
	 */
	public X500Principal getIssuerX500Principal() {
		return null;
	}

	/**
	 * Returns the subject (subject distinguished name) value from the certificate as an <code>X500Principal</code>. If
	 * the subject value is empty, then the <code>getName()</code> method of the returned <code>X500Principal</code>
	 * object returns an empty string ("").
	 * <p>
	 * It is recommended that subclasses override this method.
	 *
	 * @return an <code>X500Principal</code> representing the subject distinguished name
	 * @since 1.4
	 */
	public X500Principal getSubjectX500Principal() {
		return null;
	}

}
