/*
 * Copyright (c) 2009, 2014 IBM Corp.
 * Copyright 2018-2021 MicroEJ Corp. This file has been modified by MicroEJ Corp.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v1.0 which accompany this distribution.
 *
 * The Eclipse Public License is available at
 *    http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 *   http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *    Dave Locke - initial API and implementation and/or initial documentation
 */
package org.eclipse.paho.client.mqttv3.internal;

import java.util.Vector;

import org.eclipse.paho.client.mqttv3.IMqttActionListener;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttToken;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttPubAck;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttPubComp;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttPublish;
import org.eclipse.paho.client.mqttv3.logging.Logger;
import org.eclipse.paho.client.mqttv3.logging.LoggerFactory;

import ej.annotation.Nullable;

/**
 * Bridge between Receiver and the external API. This class gets called by Receiver, and then converts the comms-centric
 * MQTT message objects into ones understood by the external API.
 */
public class CommsCallback implements Runnable {
	private static final String CLASS_NAME = CommsCallback.class.getName();
	private static final Logger log = LoggerFactory.getLogger(LoggerFactory.MQTT_CLIENT_MSG_CAT, CLASS_NAME);

	private static final String INBOUND_QUEUE_SIZE_PROPERTY = "org.eclipse.paho.inbound.queue.size";
	private static final int INBOUND_QUEUE_SIZE_DEFAULT_VALUE = 10;
	private static final int INBOUND_QUEUE_SIZE = Integer.getInteger(INBOUND_QUEUE_SIZE_PROPERTY,
			INBOUND_QUEUE_SIZE_DEFAULT_VALUE);
	private static final String INBOUND_ACK_QUEUE_SIZE_PROPERTY = "org.eclipse.paho.inbound.ack.queue.size";
	private static final int INBOUND_ACK_QUEUE_SIZE = Integer.getInteger(INBOUND_ACK_QUEUE_SIZE_PROPERTY,
			INBOUND_QUEUE_SIZE);
	@Nullable
	private MqttCallback mqttCallback;
	@Nullable
	private final ClientComms clientComms;
	@Nullable
	private final Vector messageQueue;
	@Nullable
	private final Vector completeQueue;
	public boolean running = false;
	private boolean quiescing = false;
	private final Object lifecycle = new Object();
	@Nullable
	private Thread callbackThread;
	private final Object workAvailable = new Object();
	private final Object spaceAvailable = new Object();
	@Nullable
	private ClientState clientState;

	CommsCallback(ClientComms clientComms) {
		this.clientComms = clientComms;
		this.messageQueue = new Vector(INBOUND_QUEUE_SIZE);
		this.completeQueue = new Vector(INBOUND_ACK_QUEUE_SIZE);
		log.setResourceName(clientComms.getClient().getClientId());
	}

	public void setClientState(ClientState clientState) {
		this.clientState = clientState;
	}

	/**
	 * Starts up the Callback thread.
	 */
	public void start(String threadName) {
		synchronized (this.lifecycle) {
			if (!this.running) {
				// Praparatory work before starting the background thread.
				// For safety ensure any old events are cleared.
				this.messageQueue.clear();
				this.completeQueue.clear();

				this.running = true;
				this.quiescing = false;
				this.callbackThread = new Thread(this, threadName);
				this.callbackThread.start();
			}
		}
	}

	/**
	 * Stops the callback thread. This call will block until stop has completed.
	 */
	public void stop() {
		final String methodName = "stop";
		synchronized (this.lifecycle) {
			if (this.running) {
				// @TRACE 700=stopping
				log.fine(CLASS_NAME, methodName, 700);
				this.running = false;
				if (!Thread.currentThread().equals(this.callbackThread)) {
					try {
						synchronized (this.workAvailable) {
							// @TRACE 701=notify workAvailable and wait for run
							// to finish
							log.fine(CLASS_NAME, methodName, 701);
							this.workAvailable.notifyAll();
						}
						// Wait for the thread to finish.
						this.callbackThread.join();
					} catch (InterruptedException ex) {
					}
				}
			}
			this.callbackThread = null;
			// @TRACE 703=stopped
			log.fine(CLASS_NAME, methodName, 703);
		}
	}

	public void setCallback(MqttCallback mqttCallback) {
		this.mqttCallback = mqttCallback;
	}

	@Override
	public void run() {
		final String methodName = "run";
		while (this.running) {
			try {
				// If no work is currently available, then wait until there is some...
				try {
					synchronized (this.workAvailable) {
						if (this.running && this.messageQueue.isEmpty() && this.completeQueue.isEmpty()) {
							// @TRACE 704=wait for workAvailable
							log.fine(CLASS_NAME, methodName, 704);
							this.workAvailable.wait();
						}
					}
				} catch (InterruptedException e) {
				}

				if (this.running) {
					// Check for deliveryComplete callbacks...
					while (!this.completeQueue.isEmpty()) {
						MqttToken token;
						synchronized (this.completeQueue) {
							token = (MqttToken) this.completeQueue.elementAt(0);
							this.completeQueue.removeElementAt(0);
						}
						synchronized (this.spaceAvailable) {
							this.spaceAvailable.notifyAll();
						}
						handleActionComplete(token);
					}

					// Check for messageArrived callbacks...
					MqttPublish message = null;
					synchronized (this.messageQueue) {
						if (!this.messageQueue.isEmpty()) {
							// Note, there is a window on connect where a publish
							// could arrive before we've
							// finished the connect logic.
							message = (MqttPublish) this.messageQueue.elementAt(0);
						}
					}
					if (null != message) {
						handleMessage(message);
						synchronized (this.messageQueue) {
							this.messageQueue.removeElementAt(0);
						}
					}
				}

				if (this.quiescing) {
					this.clientState.checkQuiesceLock();
				}

			} catch (Throwable ex) {
				// Users code could throw an Error or Exception e.g. in the case
				// of class NoClassDefFoundError
				// @TRACE 714=callback threw exception
				log.fine(CLASS_NAME, methodName, 714, null, ex);
				this.running = false;
				this.clientComms.shutdownConnection(null, new MqttException(ex));

			} finally {
				synchronized (this.spaceAvailable) {
					// Notify the spaceAvailable lock, to say that there's now
					// some space on the queue...

					// @TRACE 706=notify spaceAvailable
					log.fine(CLASS_NAME, methodName, 706);
					this.spaceAvailable.notifyAll();
				}
			}
		}
	}

	private void handleActionComplete(MqttToken token) throws MqttException {
		final String methodName = "handleActionComplete";
		synchronized (token) {
			// @TRACE 705=callback and notify for key={0}
			log.fine(CLASS_NAME, methodName, 705, new Object[] { token.internalTok.getKey() });

			// Unblock any waiters and if pending complete now set completed
			token.internalTok.notifyComplete();

			if (!token.internalTok.isNotified()) {
				// If a callback is registered and delivery has finished
				// call delivery complete callback.
				if (this.mqttCallback != null && token instanceof MqttDeliveryToken && token.isComplete()) {
					this.mqttCallback.deliveryComplete((MqttDeliveryToken) token);
				}
				// Now call async action completion callbacks
				fireActionEvent(token);
			}

			// Set notified so we don't tell the user again about this action.
			if (token.isComplete()) {
				if (token instanceof MqttDeliveryToken || token.getActionCallback() instanceof IMqttActionListener) {
					token.internalTok.setNotified(true);
				}
			}

			if (token.isComplete()) {
				// Finish by doing any post processing such as delete
				// from persistent store but only do so if the action
				// is complete
				this.clientState.notifyComplete(token);
			}
		}
	}

	/**
	 * This method is called when the connection to the server is lost. If there is no cause then it was a clean
	 * disconnect. The connectionLost callback will be invoked if registered and run on the thread that requested
	 * shutdown e.g. receiver or sender thread. If the request was a user initiated disconnect then the disconnect token
	 * will be notified.
	 *
	 * @param cause
	 *            the reason behind the loss of connection.
	 */
	public void connectionLost(@Nullable MqttException cause) {
		final String methodName = "connectionLost";
		// If there was a problem and a client callback has been set inform
		// the connection lost listener of the problem.
		try {
			if (this.mqttCallback != null && cause != null) {
				// @TRACE 708=call connectionLost
				log.fine(CLASS_NAME, methodName, 708, new Object[] { cause });
				this.mqttCallback.connectionLost(cause);
			}
		} catch (java.lang.Throwable t) {
			// Just log the fact that a throwable has caught connection lost
			// is called during shutdown processing so no need to do anything else
			// @TRACE 720=exception from connectionLost {0}
			log.fine(CLASS_NAME, methodName, 720, new Object[] { t });
		}
	}

	/**
	 * An action has completed - if a completion listener has been set on the token then invoke it with the outcome of
	 * the action.
	 *
	 * @param token
	 */
	public void fireActionEvent(@Nullable MqttToken token) {
		final String methodName = "fireActionEvent";

		if (token != null) {
			IMqttActionListener asyncCB = token.getActionCallback();
			if (asyncCB != null) {
				if (token.getException() == null) {
					// @TRACE 716=call onSuccess key={0}
					log.fine(CLASS_NAME, methodName, 716, new Object[] { token.internalTok.getKey() });
					asyncCB.onSuccess(token);
				} else {
					// @TRACE 717=call onFailure key {0}
					log.fine(CLASS_NAME, methodName, 716, new Object[] { token.internalTok.getKey() });
					asyncCB.onFailure(token, token.getException());
				}
			}
		}
	}

	/**
	 * This method is called when a message arrives on a topic. Messages are only added to the queue for inbound
	 * messages if the client is not quiescing.
	 *
	 * @param sendMessage
	 *            the MQTT SEND message.
	 */
	public void messageArrived(MqttPublish sendMessage) {
		final String methodName = "messageArrived";
		if (this.mqttCallback != null) {
			// If we already have enough messages queued up in memory, wait
			// until some more queue space becomes available. This helps
			// the client protect itself from getting flooded by messages
			// from the server.
			synchronized (this.spaceAvailable) {
				while (this.running && !this.quiescing && this.messageQueue.size() >= INBOUND_QUEUE_SIZE) {
					try {
						// @TRACE 709=wait for spaceAvailable
						log.fine(CLASS_NAME, methodName, 709);
						this.spaceAvailable.wait(200);
					} catch (InterruptedException ex) {
					}
				}
			}
			if (!this.quiescing) {
				this.messageQueue.addElement(sendMessage);
				// Notify the CommsCallback thread that there's work to do...
				synchronized (this.workAvailable) {
					// @TRACE 710=new msg avail, notify workAvailable
					log.fine(CLASS_NAME, methodName, 710);
					this.workAvailable.notifyAll();
				}
			}
		}
	}

	/**
	 * Let the call back thread quiesce. Prevent new inbound messages being added to the process queue and let existing
	 * work quiesce. (until the thread is told to shutdown).
	 */
	public void quiesce() {
		final String methodName = "quiesce";
		this.quiescing = true;
		synchronized (this.spaceAvailable) {
			// @TRACE 711=quiesce notify spaceAvailable
			log.fine(CLASS_NAME, methodName, 711);
			// Unblock anything waiting for space...
			this.spaceAvailable.notifyAll();
		}
	}

	public boolean isQuiesced() {
		if (this.quiescing && this.completeQueue.size() == 0 && this.messageQueue.size() == 0) {
			return true;
		}
		return false;
	}

	private void handleMessage(MqttPublish publishMessage) throws MqttException, Exception {
		final String methodName = "handleMessage";
		// If quisecing process any pending messages.
		if (this.mqttCallback != null) {
			String destName = publishMessage.getTopicName();

			// @TRACE 713=call messageArrived key={0} topic={1}
			log.fine(CLASS_NAME, methodName, 713,
					new Object[] { new Integer(publishMessage.getMessageId()), destName });
			this.mqttCallback.messageArrived(destName, publishMessage.getMessage());
			if (publishMessage.getMessage().getQos() == 1) {
				this.clientComms.internalSend(new MqttPubAck(publishMessage),
						new MqttToken(this.clientComms.getClient().getClientId()));
			} else if (publishMessage.getMessage().getQos() == 2) {
				this.clientComms.deliveryComplete(publishMessage);
				MqttPubComp pubComp = new MqttPubComp(publishMessage);
				this.clientComms.internalSend(pubComp, new MqttToken(this.clientComms.getClient().getClientId()));
			}
		}
	}

	public void asyncOperationComplete(MqttToken token) {
		final String methodName = "asyncOperationComplete";

		if (this.running) {
			// If we already have enough messages queued up in memory, wait
			// until some more queue space becomes available. This helps
			// the client protect itself from getting flooded by messages
			// from the server.
			synchronized (this.spaceAvailable) {
				while (this.running && !this.quiescing && this.completeQueue.size() >= INBOUND_ACK_QUEUE_SIZE) {
					try {
						this.spaceAvailable.wait(200);
					} catch (InterruptedException ex) {
						// do nothing
					}
				}
			}
		}

		if (this.running) {
			// invoke callbacks on callback thread
			this.completeQueue.addElement(token);
			synchronized (this.workAvailable) {
				// @TRACE 715=new workAvailable. key={0}
				log.fine(CLASS_NAME, methodName, 715, new Object[] { token.internalTok.getKey() });
				this.workAvailable.notifyAll();
			}
		} else {
			// invoke async callback on invokers thread
			try {
				handleActionComplete(token);
			} catch (Throwable ex) {
				// Users code could throw an Error or Exception e.g. in the case
				// of class NoClassDefFoundError
				// @TRACE 719=callback threw ex:
				log.fine(CLASS_NAME, methodName, 719, null, ex);

				// Shutdown likely already in progress but no harm to confirm
				this.clientComms.shutdownConnection(null, new MqttException(ex));
			}

		}
	}

	/**
	 * Returns the thread used by this callback.
	 */
	@Nullable
	protected Thread getThread() {
		return this.callbackThread;
	}
}
