/*
 * Copyright (c) 2009, 2014 IBM Corp.
 * Copyright 2018-2021 MicroEJ Corp. This file has been modified by MicroEJ Corp.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v1.0 which accompany this distribution.
 *
 * The Eclipse Public License is available at
 *    http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 *   http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *    Dave Locke - initial API and implementation and/or initial documentation
 */
package org.eclipse.paho.client.mqttv3.internal;

import java.io.IOException;
import java.io.OutputStream;

import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttToken;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttAck;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttDisconnect;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttOutputStream;
import org.eclipse.paho.client.mqttv3.internal.wire.MqttWireMessage;
import org.eclipse.paho.client.mqttv3.logging.Logger;
import org.eclipse.paho.client.mqttv3.logging.LoggerFactory;

import ej.annotation.Nullable;

public class CommsSender implements Runnable {
	private static final String CLASS_NAME = CommsSender.class.getName();
	private static final Logger log = LoggerFactory.getLogger(LoggerFactory.MQTT_CLIENT_MSG_CAT, CLASS_NAME);

	// Sends MQTT packets to the server on its own thread
	private boolean running = false;
	private final Object lifecycle = new Object();
	@Nullable
	private ClientState clientState = null;
	@Nullable
	private final MqttOutputStream out;
	@Nullable
	private ClientComms clientComms = null;
	@Nullable
	private CommsTokenStore tokenStore = null;
	@Nullable
	private Thread sendThread = null;

	public CommsSender(ClientComms clientComms, ClientState clientState, CommsTokenStore tokenStore, OutputStream out) {
		this.out = new MqttOutputStream(clientState, out);
		this.clientComms = clientComms;
		this.clientState = clientState;
		this.tokenStore = tokenStore;
		log.setResourceName(clientComms.getClient().getClientId());
	}

	/**
	 * Starts up the Sender thread.
	 */
	public void start(String threadName) {
		synchronized (this.lifecycle) {
			if (!this.running) {
				this.running = true;
				this.sendThread = new Thread(this, threadName);
				this.sendThread.start();
			}
		}
	}

	/**
	 * Stops the Sender's thread. This call will block.
	 */
	public void stop() {
		final String methodName = "stop";

		synchronized (this.lifecycle) {
			// @TRACE 800=stopping sender
			log.fine(CLASS_NAME, methodName, 800);
			if (this.running) {
				this.running = false;
				if (!Thread.currentThread().equals(this.sendThread)) {
					try {
						// first notify get routine to finish
						this.clientState.notifyQueueLock();
						// Wait for the thread to finish.
						this.sendThread.join();
					} catch (InterruptedException ex) {
					}
				}
			}
			this.sendThread = null;
			// @TRACE 801=stopped
			log.fine(CLASS_NAME, methodName, 801);
		}
	}

	@Override
	public void run() {
		final String methodName = "run";
		MqttWireMessage message = null;
		while (this.running && (this.out != null)) {
			try {
				message = this.clientState.get();
				if (message != null) {
					// @TRACE 802=network send key={0} msg={1}
					log.fine(CLASS_NAME, methodName, 802, new Object[] { message.getKey(), message });

					if (message instanceof MqttAck) {
						this.out.write(message);
						this.out.flush();
					} else {
						MqttToken token = this.tokenStore.getToken(message);
						// While quiescing the tokenstore can be cleared so need
						// to check for null for the case where clear occurs
						// while trying to send a message.
						if (token != null) {
							synchronized (token) {
								this.out.write(message);
								try {
									this.out.flush();
								} catch (IOException ex) {
									// The flush has been seen to fail on disconnect of a SSL socket
									// as disconnect is in progress this should not be treated as an error
									if (!(message instanceof MqttDisconnect)) {
										throw ex;
									}
								}
								this.clientState.notifySent(message);
							}
						}
					}
				} else { // null message
					// @TRACE 803=get message returned null, stopping}
					log.fine(CLASS_NAME, methodName, 803);

					this.running = false;
				}
			} catch (MqttException me) {
				handleRunException(message, me);
			} catch (Exception ex) {
				handleRunException(message, ex);
			}
		} // end while

		// @TRACE 805=<
		log.fine(CLASS_NAME, methodName, 805);

	}

	private void handleRunException(MqttWireMessage message, Exception ex) {
		final String methodName = "handleRunException";
		// @TRACE 804=exception
		log.fine(CLASS_NAME, methodName, 804, null, ex);
		MqttException mex;
		if (!(ex instanceof MqttException)) {
			mex = new MqttException(MqttException.REASON_CODE_CONNECTION_LOST, ex);
		} else {
			mex = (MqttException) ex;
		}

		this.running = false;
		this.clientComms.shutdownConnection(null, mex);
	}
}
