/*
 * Copyright (c) 2009, 2014 IBM Corp.
 * Copyright 2018-2021 MicroEJ Corp. This file has been modified by MicroEJ Corp.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v1.0 which accompany this distribution.
 *
 * The Eclipse Public License is available at
 *    http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 *   http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *    Dave Locke - initial API and implementation and/or initial documentation
 */
package org.eclipse.paho.client.mqttv3.internal.wire;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.internal.ClientState;
import org.eclipse.paho.client.mqttv3.logging.Logger;
import org.eclipse.paho.client.mqttv3.logging.LoggerFactory;

import ej.annotation.Nullable;

/**
 * An <code>MqttOutputStream</code> lets applications write instances of <code>MqttWireMessage</code>.
 */
public class MqttOutputStream extends OutputStream {
	private static final String CLASS_NAME = MqttOutputStream.class.getName();
	private static final Logger log = LoggerFactory.getLogger(LoggerFactory.MQTT_CLIENT_MSG_CAT, CLASS_NAME);

	@Nullable
	private ClientState clientState = null;
	private final BufferedOutputStream out;

	public MqttOutputStream(ClientState clientState, OutputStream out) {
		this.clientState = clientState;
		this.out = new BufferedOutputStream(out);
	}

	@Override
	public void close() throws IOException {
		this.out.close();
	}

	@Override
	public void flush() throws IOException {
		this.out.flush();
	}

	@Override
	public void write(byte[] b) throws IOException {
		this.out.write(b);
		this.clientState.notifySentBytes(b.length);
	}

	@Override
	public void write(byte[] b, int off, int len) throws IOException {
		this.out.write(b, off, len);
		this.clientState.notifySentBytes(len);
	}

	@Override
	public void write(int b) throws IOException {
		this.out.write(b);
	}

	/**
	 * Writes an <code>MqttWireMessage</code> to the stream.
	 */
	public void write(MqttWireMessage message) throws IOException, MqttException {
		final String methodName = "write";
		byte[] bytes = message.getHeader();
		byte[] pl = message.getPayload();
		// out.write(message.getHeader());
		// out.write(message.getPayload());
		this.out.write(bytes, 0, bytes.length);
		this.clientState.notifySentBytes(bytes.length);

		int offset = 0;
		int chunckSize = 1024;
		while (offset < pl.length) {
			int length = Math.min(chunckSize, pl.length - offset);
			this.out.write(pl, offset, length);
			offset += chunckSize;
			this.clientState.notifySentBytes(length);
		}

		// @TRACE 500= sent {0}
		log.fine(CLASS_NAME, methodName, 500, new Object[] { message });
	}
}
