/*
 * Copyright (c) 2009, 2014 IBM Corp.
 * Copyright 2018-2021 MicroEJ Corp. This file has been modified by MicroEJ Corp.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v1.0 which accompany this distribution.
 *
 * The Eclipse Public License is available at
 *    http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 *   http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *    Dave Locke - initial API and implementation and/or initial documentation
 */
package org.eclipse.paho.client.mqttv3.internal.wire;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

import ej.annotation.Nullable;

/**
 * An on-the-wire representation of an MQTT SEND message.
 */
public class MqttPublish extends MqttPersistableWireMessage {

	private final MqttMessage message;
	private final String topicName;

	@Nullable
	private byte[] encodedPayload = null;

	public MqttPublish(String name, MqttMessage message) {
		super(MqttWireMessage.MESSAGE_TYPE_PUBLISH);
		this.topicName = name;
		this.message = message;
	}

	/**
	 * Constructs a new MqttPublish object.
	 *
	 * @param info
	 *            the message info byte
	 * @param data
	 *            the variable header and payload bytes
	 */
	public MqttPublish(byte info, byte[] data) throws MqttException, IOException {
		super(MqttWireMessage.MESSAGE_TYPE_PUBLISH);
		this.message = new MqttReceivedMessage();
		this.message.setQos((info >> 1) & 0x03);
		if ((info & 0x01) == 0x01) {
			this.message.setRetained(true);
		}
		if ((info & 0x08) == 0x08) {
			((MqttReceivedMessage) this.message).setDuplicate(true);
		}

		ByteArrayInputStream bais = new ByteArrayInputStream(data);
		CountingInputStream counter = new CountingInputStream(bais);
		DataInputStream dis = new DataInputStream(counter);
		this.topicName = decodeUTF8(dis);
		if (this.message.getQos() > 0) {
			this.msgId = dis.readUnsignedShort();
		}
		byte[] payload = new byte[data.length - counter.getCounter()];
		dis.readFully(payload);
		dis.close();
		this.message.setPayload(payload);
	}

	@Override
	public String toString() {

		// Convert the first few bytes of the payload into a hex string
		StringBuffer hex = new StringBuffer();
		byte[] payload = this.message.getPayload();
		int limit = Math.min(payload.length, 20);
		for (int i = 0; i < limit; i++) {
			byte b = payload[i];
			String ch = Integer.toHexString(b);
			if (ch.length() == 1) {
				ch = "0" + ch;
			}
			hex.append(ch);
		}

		// It will not always be possible to convert the binary payload into
		// characters, but never-the-less we attempt to do this as it is often
		// useful
		String string = null;
		try {
			string = new String(payload, 0, limit, "UTF-8");
		} catch (Exception e) {
			string = "?";
		}

		StringBuffer sb = new StringBuffer();
		sb.append(super.toString());
		sb.append(" qos:").append(this.message.getQos());
		if (this.message.getQos() > 0) {
			sb.append(" msgId:").append(this.msgId);
		}
		sb.append(" retained:").append(this.message.isRetained());
		sb.append(" dup:").append(this.duplicate);
		sb.append(" topic:\"").append(this.topicName).append("\"");
		sb.append(" payload:[hex:").append(hex);
		sb.append(" utf8:\"").append(string).append("\"");
		sb.append(" length:").append(payload.length).append("]");

		return sb.toString();
	}

	@Override
	protected byte getMessageInfo() {
		byte info = (byte) (this.message.getQos() << 1);
		if (this.message.isRetained()) {
			info |= 0x01;
		}
		if (this.message.isDuplicate() || this.duplicate) {
			info |= 0x08;
		}

		return info;
	}

	public String getTopicName() {
		return this.topicName;
	}

	public MqttMessage getMessage() {
		return this.message;
	}

	protected static byte[] encodePayload(MqttMessage message) {
		return message.getPayload();
	}

	@Override
	public byte[] getPayload() throws MqttException {
		if (this.encodedPayload == null) {
			this.encodedPayload = encodePayload(this.message);
		}
		return this.encodedPayload;
	}

	@Override
	public int getPayloadLength() {
		int length = 0;
		try {
			length = getPayload().length;
		} catch (MqttException me) {
		}
		return length;
	}

	@Override
	public void setMessageId(int msgId) {
		super.setMessageId(msgId);
		if (this.message instanceof MqttReceivedMessage) {
			((MqttReceivedMessage) this.message).setMessageId(msgId);
		}
	}

	@Override
	protected byte[] getVariableHeader() throws MqttException {
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			DataOutputStream dos = new DataOutputStream(baos);
			encodeUTF8(dos, this.topicName);
			if (this.message.getQos() > 0) {
				dos.writeShort(this.msgId);
			}
			dos.flush();
			return baos.toByteArray();
		} catch (IOException ex) {
			throw new MqttException(ex);
		}
	}

	@Override
	public boolean isMessageIdRequired() {
		// all publishes require a message ID as it's used as the key to the token store
		return true;
	}
}